package jist.swans.route;

import jist.swans.mac.MacAddress;
import jist.swans.net.NetAddress;
import jist.swans.net.NetMessage;
import jist.runtime.JistAPI;
import jist.swans.Constants;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

public class RouteDhg extends RouteDsr implements RouteInterface.Dhg {

	/** true if is a AP*/
	private boolean AP;
	private RouteInterface.Dhg self;
	
	/** TTL for one cellule */
	private byte diameter;
	
	/**AP selected */
	private APTable.APEntry apSelected;
	
	
	/** time between two advertissement */
	private long advertissementInter=Constants.SECOND*5;
	
	/** table of APs*/
	APTable apTable;
	
	
	/**
	 * AP's Table (set of APEntry
	 * 
	 * @author florent
	 *
	 */
	private class APTable{
		/**
		 * Entry for one AP in a MN
		 * 
		 * @author florent
		 *
		 */
		private class APEntry{
			/** AP's address*/
			NetAddress addr;
			/** number of hop to the AP*/
			byte hop;
			/** add's time */
			long time;
			
			/** RouteRequest option of AP Advertissement 
			 * (used for get the route to the AP) */
			RouteDsrMsg.OptionRouteRequest rReq;

			
			APEntry(NetAddress addr,byte hop,long time,RouteDsrMsg.OptionRouteRequest rReq){
				this.addr=addr;
				this.hop=hop;
				this.time=time;
				this.rReq=rReq;
			}
		}
		
		HashSet APs;
		
		void add(NetAddress addr,byte hop,RouteDsrMsg.OptionRouteRequest rReq){
			Iterator it=APs.iterator();
			
			while(it.hasNext()){
				APEntry ap=(APEntry)it.next();
				
				if(ap.addr.equals(addr))
					it.remove();
		
			}
			
			
			
			APs.add(new APEntry(addr,hop,JistAPI.getTime(),rReq));
		}
		
		/**
		 * Compute the nearer AP
		 *
		 */
		 APEntry computeNearerAP(){
			Iterator it=APs.iterator();
			APEntry apSelected=null;
			
			while(it.hasNext()){
				APEntry ap=(APEntry)it.next();
				
				//if the minimum hop and AP advertissement fresh )
				if((apSelected==null || apSelected.hop>ap.hop) && ((JistAPI.getTime()-ap.time)<(2*advertissementInter)))
					apSelected=ap;
			}
			
			return apSelected;
		}
		
	}
	

	
	
	public RouteDhg(NetAddress localAddr){
		this(localAddr,false);
		
	}
	
	public RouteDhg(NetAddress localAddr,boolean AP){
		super(localAddr);	
		this.AP=AP;
		self = (RouteInterface.Dhg)JistAPI.proxy(this, RouteInterface.Dhg.class);
		diameter = 8;
		if(AP)
			self.RequestPosTimeOut();
	}
	
	private void registreToAP(NetAddress ap,NetAddress src,RouteDsrMsg.OptionRouteRequest opt){
	    NetAddress[] routeToHere = new NetAddress[opt.getNumAddresses() + 2];
	    routeToHere[0] = src;
	    for (int i = 1; i < routeToHere.length-1; i++)
	    {
	      routeToHere[i] = opt.getAddress(i-1);
	    }
	    
	    routeToHere[routeToHere.length - 1] = localAddr;
	    
	    NetAddress[] routeFromHere = new NetAddress[routeToHere.length - 2];
	    for (int i = 0; i < routeFromHere.length; i++)
	    {
	      routeFromHere[i] = routeToHere[routeToHere.length - i - 2];
	    }
	    
	    // Add a Route Reply option indicating how to get here from the
	    // source and a Source Route option indicating how to get to the
	    // source from here.
	    RouteDsrMsg reply = new RouteDsrMsg(null);
	    reply.addOption(RouteDsrMsg.OptionRouteReply.create(routeToHere));
	    
	    
	    
	    if (routeFromHere.length > 0)
	    {
	      reply.addOption(RouteDsrMsg.OptionSourceRoute.create(0,
	        routeFromHere.length, routeFromHere));
	    }
	    
	    reply.addOption(RouteDsrMsg.OptionAccessPointRegistry.create());
	    
    

	    NetMessage.Ip replyMsg = new NetMessage.Ip(reply, localAddr,
	      src, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
	      Constants.TTL_DEFAULT);
	    

	    JistAPI.sleep((long)(Math.random() * BROADCAST_JITTER));
	    Transmit(replyMsg);
		
	}
	
	
	private void HandleAPAdvertissement(RouteDsrMsg msg,RouteDsrMsg.OptionRouteRequest optRReq,RouteDsrMsg.OptionAccessPointAdvertissement optAPAdv,NetAddress src){
		int hop=optRReq.getNumAddresses()+2;
		apTable.add(src, (byte)hop,optRReq);
		
	 	APTable.APEntry apCanditate=apTable.computeNearerAP();
	 	
	 	//not AP accesible
	 	if(apCanditate==null){
	 		return;
	 	}
		
		if(apSelected==null ||  (apSelected.hop-apCanditate.hop) >= 2 ){
			apSelected=apCanditate;
			registreToAP(apSelected.addr,localAddr,apSelected.rReq);
		}
			
			
	}
	
	  protected void ProcessOptions(RouteDsrMsg msg, NetAddress src, NetAddress dst,
              short protocol, byte priority, byte ttl,
              short id, short fragOffset){
		  
		   Iterator iter = msg.getOptions().iterator();
		   
		   //get the RouteRequest opiton for the AP advertissement option.
		   RouteDsrMsg.OptionRouteRequest optRReq=null;

		    while (iter.hasNext())
		    {
		      byte[] optBuf = (byte[])iter.next();
		      RouteDsrMsg.Option opt = RouteDsrMsg.Option.create(optBuf, 0);

		      if (opt == null)
		      {
		        // This should never happen in the simulation
		        throw new RuntimeException("Unrecognized DSR Option");
		      }

		      switch (opt.getType())
		      {
		       case RouteDsrMsg.OPT_ROUTE_REQUEST:
		    	   optRReq=(RouteDsrMsg.OptionRouteRequest)opt;
		  		  break;
		      
		        case RouteDsrMsg.OPT_AP_ADVERTISSEMENT:
		        	if(optRReq==null)
		        		throw new RuntimeException("AP advertissement without RouteRequest");
		        	
		        	HandleAPAdvertissement(msg,optRReq,(RouteDsrMsg.OptionAccessPointAdvertissement)opt,src);
		          break;

			    }
		      
		    }


		  //add forward condition
		  
		  super.ProcessOptions(msg,src,dst,protocol,priority,ttl,id,fragOffset);
	  }
	
	
	protected void broadcastAdvertissement(){

	
	      RouteDsrMsg routeRequest = new RouteDsrMsg(null);
	      routeRequest.addOption(RouteDsrMsg.OptionRouteRequest.create(nextRequestId++,
	    		  NetAddress.ANY, new NetAddress[0]));
	      
	      routeRequest.addOption(RouteDsrMsg.OptionAccessPointAdvertissement.create());

	      NetMessage.Ip routeRequestMsg = new NetMessage.Ip(routeRequest, localAddr,
	        NetAddress.ANY, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
	        diameter);
	      

	      
	      netEntity.send(routeRequestMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
		
		
		
	}
	
    public void RequestPosTimeOut(){
    	broadcastAdvertissement();
    	JistAPI.sleep(advertissementInter);
    }

	public long getAdvertissementInter() {
		return advertissementInter;
	}

	public void setAdvertissementInter(long advertissementInter) {
		this.advertissementInter = advertissementInter;
	}
}
