package jist.swans.runway.graph;


import jargs.gnu.CmdLineParser;

import java.util.*;
import java.util.Random;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JPanel;


import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.graph.impl.*;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.ISOMLayout;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.ShapePickSupport;
import edu.uci.ics.jung.visualization.SpringLayout;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.contrib.CircleLayout;
import edu.uci.ics.jung.visualization.contrib.KKLayout;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;
import edu.uci.ics.jung.algorithms.shortestpath.*;


import jist.swans.runway.*;
import jist.swans.misc.*;


public class Graph {
	
	
	public static final float DIST=250.0f;
	//list of vehicle nodes
	public LinkedList<CarVertex> l;
	//list of AP
	public LinkedList<APVertex> l_ap;		
	private edu.uci.ics.jung.graph.Graph g;
	private RunWay highway;
	private double dAP;
	private int nbAP;

	
	public Graph(RunWay highway,double dAP,int nbAP){
		this.highway = highway;
		this.dAP=dAP;
		this.nbAP=nbAP;
		init();
		updateGraphe();
	}
	
	private void init(){
		
		l= new LinkedList<CarVertex>();
		g = new  DirectedSparseGraph();
		
		for (int i=0;i<highway.getNbrLan();i++){
			for (RunWayCar car=highway.beginEnumCar(i);car!=null;car=car.next){
				CarVertex c=new CarVertex(car);
				l.add(c);
				g.addVertex(c);
			}
		}	
		
		l_ap = new LinkedList<APVertex>();
		//add th APs
		float inter = 1000.0f / (float)dAP;
		
		for(int i=0;i<nbAP;i++){
			APVertex c=new APVertex(i*(-1),(i+1)*inter,highway);
			l.add(c);
			l_ap.add(c);
			g.addVertex(c);			  
		}
	}
	

	
	public void updateGraphe(){
		
		Collections.sort(l);
		g.removeAllEdges();
		Iterator<CarVertex> iterator=l.iterator();

		int i=0;
		while(iterator.hasNext()){
			CarVertex c=iterator.next();
			
			float loc_min = c.getLoc()-DIST;
			float loc_max = c.getLoc()+DIST;
			
			//System.out.print(c + " : ");
			
			
			ListIterator<CarVertex> neighbord=l.listIterator(i+1);
			while(neighbord.hasNext()){
				CarVertex n=neighbord.next();

				if (n.getLoc()>loc_max)
					break;
				//System.out.println(n.getLoc()-c.getLoc());	
				g.addEdge(new DirectedSparseEdge(c,n));
			}
			//System.out.print(" |||| ");
			neighbord=l.listIterator(i);
			while(neighbord.hasPrevious()){
				CarVertex n=neighbord.previous();

				if (n.getLoc()<loc_min)
					break;
					
				//System.out.println(n.getLoc()-c.getLoc());
				g.addEdge(new DirectedSparseEdge(c,n));
			}			
			//System.out.println();
			i++;
		}
		
		//link the APs
		Iterator<APVertex> ap_iterator=l_ap.iterator();


		
		if(ap_iterator.hasNext()){

		
			APVertex ap_prec=ap_iterator.next();
		
			while(ap_iterator.hasNext()){
				APVertex ap=ap_iterator.next();
				
				g.addEdge(new DirectedSparseEdge(ap_prec,ap));
				g.addEdge(new DirectedSparseEdge(ap,ap_prec));				
				
				ap_prec=ap;
			}
		
		}
		//System.out.println(l);
		
		/*DijkstraShortestPath sp = new DijkstraShortestPath(g);
		
		List path = sp.getPath(l.get(5), l.get(50));
		System.out.println(path);*/
		
		
		//UnweightedShortestPath();		
	}
	
	public edu.uci.ics.jung.graph.Graph getGraphe(){
		return g;
	}
	
	public String toString(){
		return l.toString();
	}
	
	
	  public static void main(String[] args) throws CmdLineParser.OptionException
	  {
		  System.err.println("Pas de test unitaire !!");
	  }
}
