package jist.swans.runway.graph;

import java.util.*;

import jist.swans.runway.*;
import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.graph.impl.*;
import edu.uci.ics.jung.algorithms.shortestpath.DijkstraShortestPath;

public class PathMonitor {
	Set<Path> paths;
	edu.uci.ics.jung.graph.Graph graph;
	
	static public float lifeTime(CarVertex  c1,CarVertex c2){
		float d = c2.getLoc()-c1.getLoc();
		
		if (d<0){
			CarVertex t=c1;
			c1=c2;
			c2=t;
			d = c2.getLoc()-c1.getLoc();
		}
		
		if (d<0)
			System.out.println("IMPOSSIBLE");
		
		float v = c2.getSpeed() - c1.getSpeed();
		
		
		if(v>0){
			return (Graph.DIST - d) / v;
		}else if (v<0){
			return (Graph.DIST + d) / -v;
		}
		
		return -1;
	}
	
	
	private class Path{
		
		public Path(CarVertex src,CarVertex dst,List<CarVertex> path){
			this.src = src;
			this.dst = dst;
			nbBreak=0;
			nbPathNoFind=-1; //la premiere fois compte pas
			sommeLongueur=0;
			nbChemin=0;
			setPath(path);
		}
		
		private CarVertex src;
		private CarVertex dst;		
		private List<CarVertex> path;
		private int nbBreak;
		private int nbPathNoFind;
		
		private int sommeLongueur;
		private int nbChemin;		
		
		public String toString(){
			if(path==null)
				return "Pas de chemin entre "+src+" et "+dst;
			else
				return "Chemin entre "+src+" et "+dst+" : "+path+ "\n\tRompu "+nbBreak+" fois"+  "\n\tNon trouvé "   + nbPathNoFind    +"\n\tLongueur : "+path.size();
		}
		
		
		private void showLifeTime(){
		
			if(path==null)
				return ;		
			
			Iterator<CarVertex> i = path.iterator();
			CarVertex v1,v2;
		
			float min_lifetime= Integer.MAX_VALUE ;
			
			if(!i.hasNext())
				return ;
			
//			get the first CarVertex (source)
			v1 = i.next();
			
			while(i.hasNext()){
				//a succesor
				v2=i.next();
				//+" " +(v1.getSpeed()-v2.getSpeed())
				//System.out.println(lifeTime(v1,v2)/*(v1.getSpeed()-v2.getSpeed())*//*,v1.toString() + v2.toString() + "\t"+ lifeTime(v1,v2) + "\t" + v1.getLoc()+" "+v2.getLoc()+" "+v1.getSpeed()+" "+v2.getSpeed()*/);
				float t=lifeTime(v1,v2);
				
				if(t<min_lifetime)
					min_lifetime=t;
				
				
				v1=v2;
			}
			
			System.out.println(path.size()+" "+min_lifetime);
		}
		
		private void showRelativSpeed(){
			

			if(path==null)
				return ;		
			
			Iterator<CarVertex> i = path.iterator();
			CarVertex v1,v2;
		
			
			if(!i.hasNext())
				return ;
			
//			get the first CarVertex (source)
			v1 = i.next();
			
			while(i.hasNext()){
				//a succesor
				v2=i.next();
				//+" " +(v1.getSpeed()-v2.getSpeed())
				//System.out.println(lifeTime(v1,v2)/*(v1.getSpeed()-v2.getSpeed())*//*,v1.toString() + v2.toString() + "\t"+ lifeTime(v1,v2) + "\t" + v1.getLoc()+" "+v2.getLoc()+" "+v1.getSpeed()+" "+v2.getSpeed()*/);
				
				v1=v2;
			}
			

			//System.out.println();
			
		}
		
		private boolean checkPath(List<CarVertex> path){

			if(path==null)
				return false;		
			
			Iterator<CarVertex> i = path.iterator();
			CarVertex v1,v2;
		
			
			if(!i.hasNext())
				return true;
			
//			get the first CarVertex (source)
			v1 = i.next();
			
			while(i.hasNext()){
				//a succesor
				v2=i.next();
				
				//v2 is always successor of v1 ?
				if(!v1.isPredecessorOf(v2)){
					
					//System.out.println("Lien rompu : "+v1+" "+v2);
					return false;
				}
				
				v1=v2;
			}
			
			return true;
		}
		
		boolean checkPath(){
			
			if(path==null){
				nbPathNoFind++;
				return false;
			}
			
			if(!checkPath(path)){
				//System.out.println(path.size());
				path=null;
				nbBreak++;
				
				return false;
			}
			
			
			
			return true;
		}
		
		public double getAvgLength(){
			return (double)sommeLongueur/(double)nbChemin;
		}

		public int getNbBreak() {
			return nbBreak;
		}

		public List<CarVertex> getPath() {
			return path;
		}

		public void setPath(List<CarVertex> path) {
			this.path = path;
			
			if(path!=null && nbChemin==0){
				nbChemin++;
				sommeLongueur+=path.size();
			}
		}

		public CarVertex getDst() {
			return dst;
		}

		public CarVertex getSrc() {
			return src;
		}

		public int getNbPathNoFind() {
			return nbPathNoFind;
		}			
	}
	
	public PathMonitor(edu.uci.ics.jung.graph.Graph graph,int nbCheminInit){
		this.graph = graph;
		
		paths = new HashSet<Path>(nbCheminInit);
	}
	
	private List<CarVertex> getShortestPath(CarVertex s,CarVertex d){
		  Edge e=null;
		
		  //get the shortest path between s and d
		  DijkstraShortestPath sp = new DijkstraShortestPath(graph);
		  List<Edge> l= sp.getPath(s, d);
		  
		  //convert the path on a list of CarVertex
		  List<CarVertex> path = new LinkedList<CarVertex>();
		  
		  Iterator<Edge> i=l.iterator();
		  
		  if(!i.hasNext())
			  return null;
		  
		  while(i.hasNext()){
			  e=i.next();
			  path.add((CarVertex)e.getEndpoints().getFirst());
		  }
		  
		  path.add((CarVertex)e.getEndpoints().getSecond());
		  
		  return path;
	}
	
	
	public void addPath(CarVertex s,CarVertex d){
		
		paths.add(new Path(s,d,null));

	}
	

	
	public void checkAllPath(){
		Iterator<Path> i=paths.iterator();
		
		int c=0;
		
		while(i.hasNext()){
			Path p=i.next();
			//System.err.println(c++);
			
			if(!p.checkPath()){
				p.setPath(getShortestPath(p.getSrc(),p.getDst()));
				
				
				//System.out.println(p);
			}
			//p.showRelativSpeed();
			//p.showLifeTime();
		}
			
	}
	
	public void showStat(){
		Iterator<Path> i=paths.iterator();
		int j=0;
		while(i.hasNext()){
			Path p=i.next();
			j++;
			
			System.out.println(j+" "+p.getAvgLength()+" "+p.getNbBreak()+" "+p.getNbPathNoFind());
			
		}
		
	}
	
	public String toString(){
		return paths.toString();
	}
}
