//////////////////////////////////////////////////
// JIST (Java In Simulation Time) Project
// Timestamp: <dsrtest.java Tue 2004/04/06 11:57:46 barr pompom.cs.cornell.edu>
//

// Copyright (C) 2004 by Cornell University
// All rights reserved.
// Refer to LICENSE for terms and conditions of use.

package driver;

import jist.runtime.JistAPI;
import jist.swans.Constants;
import jist.swans.misc.Util;
import jist.swans.misc.Mapper;
import jist.swans.misc.Location;
import jist.swans.field.Fading;
import jist.swans.field.Field;
import jist.swans.field.Mobility;
import jist.swans.field.PathLoss;
import jist.swans.field.Spatial;
import jist.swans.radio.RadioNoise;
import jist.swans.radio.RadioNoiseIndep;
import jist.swans.radio.RadioInfo;
import jist.swans.bus.BusEthernetSwitch;
import jist.swans.mac.MacAddress;
import jist.swans.mac.Mac802_11;
import jist.swans.mac.MacDumbBus;
import jist.swans.net.NetAddress;
import jist.swans.net.NetIp;
import jist.swans.net.NetMultiInterface;
import jist.swans.net.PacketLoss;
import jist.swans.trans.TransUdp;
import jist.swans.app.AppJava;
import jist.swans.route.RouteDsr;

import java.net.InetAddress;
import java.net.DatagramSocket;
import java.net.DatagramPacket;

public class dsrtest
{

  public static final byte[] SERVER_IP = {0, 0, 0, 6};
  public static final int PORT = 3001;

  public static class Server
  {
    public static void main(String args[])
    {
      try
      {
        System.out.println("server starting at t="+JistAPI.getTime());
        DatagramSocket socket = new DatagramSocket(PORT);
        byte buf[] = new byte[256];
        DatagramPacket packet = new DatagramPacket(buf, buf.length);
        socket.receive(packet);
        socket.close();
        System.out.println("received at t="+JistAPI.getTime()+
            " ("+packet.getLength()+" bytes) "
            +(new String(packet.getData(), packet.getOffset(), packet.getLength())));
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }

  public static class Client
  {
    public static void main(String args[])
    {
      try
      {
        System.out.println("client starting at t="+JistAPI.getTime());
        DatagramSocket socket = new DatagramSocket();
        byte[] buf = "hi".getBytes();
        DatagramPacket packet = new DatagramPacket(
          buf,
          buf.length,
          InetAddress.getByAddress(SERVER_IP),
          PORT);

        System.out.println("sent at t="+JistAPI.getTime());
        System.out.flush();
        socket.send(packet);
        socket.close();
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }

  private static void TwoNodeTest()
  {
   try
    {
      // field
      Location bounds = new Location.Location2D(200, 200);
      Field field = new Field(bounds);
      RadioInfo.RadioInfoShared radioInfoShared = RadioInfo.createShared(
          Constants.FREQUENCY_DEFAULT, 
          Constants.BANDWIDTH_DEFAULT,
          Constants.TRANSMIT_DEFAULT, 
          Constants.GAIN_DEFAULT,
          Util.fromDB(Constants.SENSITIVITY_DEFAULT), 
          Util.fromDB(Constants.THRESHOLD_DEFAULT),
          Constants.TEMPERATURE_DEFAULT, 
          Constants.TEMPERATURE_FACTOR_DEFAULT, 
          Constants.AMBIENT_NOISE_DEFAULT);
      // protocol mapper
      Mapper protMap = new Mapper(Constants.NET_PROTOCOL_MAX);
      protMap.mapToNext(Constants.NET_PROTOCOL_DSR);
      protMap.mapToNext(Constants.NET_PROTOCOL_UDP);
      // packet loss
      PacketLoss pl = new PacketLoss.Zero();
      // nodes
      int i=1;
      // radio
      RadioNoise radio1 = new RadioNoiseIndep(i, radioInfoShared);
      RadioNoise radio2 = new RadioNoiseIndep(i+1, radioInfoShared);
      // mac
      Mac802_11 mac1 = new Mac802_11(new MacAddress(i), radio1.getRadioInfo());
      Mac802_11 mac2 = new Mac802_11(new MacAddress(i+1), radio2.getRadioInfo());
      // net
      NetIp net1 = new NetIp(new NetAddress(SERVER_IP), protMap, pl, pl);
      NetIp net2 = new NetIp(new NetAddress(i), protMap, pl, pl);
      // route
      RouteDsr dsr1 = new RouteDsr(net1.getAddress());
      RouteDsr dsr2 = new RouteDsr(net2.getAddress());
      // trans
      TransUdp udp1 = new TransUdp();
      TransUdp udp2 = new TransUdp();

      // field hookup
      Location location1 = new Location.Location2D(0, 0);
      Location location2 = new Location.Location2D(0, 1);
      field.addRadio(radio1.getRadioInfo(), radio1.getProxy(), location1);
      field.addRadio(radio2.getRadioInfo(), radio2.getProxy(), location2);
      // radio hookup
      radio1.setFieldEntity(field.getProxy());
      radio2.setFieldEntity(field.getProxy());
      radio1.setMacEntity(mac1.getProxy());
      radio2.setMacEntity(mac2.getProxy());
      // route hookup
      dsr1.setNetEntity(net1.getProxy());
      dsr2.setNetEntity(net2.getProxy());
      // net hookup
      byte intId1 = net1.addInterface(mac1.getProxy());
      byte intId2 = net2.addInterface(mac2.getProxy());
      net1.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr2.getProxy());
      net1.setRouting(dsr1.getProxy());
      net2.setRouting(dsr2.getProxy());
      // mac hookup
      mac1.setRadioEntity(radio1.getProxy());
      mac2.setRadioEntity(radio2.getProxy());
      mac1.setNetEntity(net1.getProxy(), intId1);
      mac2.setNetEntity(net2.getProxy(), intId2);
      // trans hookup
      udp1.setNetEntity(net1.getProxy());
      udp2.setNetEntity(net2.getProxy());
      net1.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp2.getProxy());

      // applications
      AppJava server = new AppJava(Server.class);
      server.setUdpEntity(udp1.getProxy());
      AppJava client = new AppJava(Client.class);
      client.setUdpEntity(udp2.getProxy());

      // run apps
      server.getProxy().run(null); 
      JistAPI.sleep(1);
      client.getProxy().run(null);
    }
    catch(Exception e) { e.printStackTrace(); }
  }

  private static void ThreeNodeTest()
  {
   try
    {
      // field
      Location bounds = new Location.Location2D(200, 2500);
      Field field = new Field(bounds);

      Field fieldWired = new Field((Spatial)new Spatial.LinearList(bounds),
            new Fading.None(), 
            new PathLoss.FreeSpace(), 
            new Mobility.Static(),
            Double.MIN_VALUE);      
      
      RadioInfo.RadioInfoShared radioInfoShared = RadioInfo.createShared(
          Constants.FREQUENCY_DEFAULT, 
          Constants.BANDWIDTH_DEFAULT,
          Constants.TRANSMIT_DEFAULT, 
          Constants.GAIN_DEFAULT,
          Util.fromDB(Constants.SENSITIVITY_DEFAULT), 
          Util.fromDB(Constants.THRESHOLD_DEFAULT),
          Constants.TEMPERATURE_DEFAULT, 
          Constants.TEMPERATURE_FACTOR_DEFAULT, 
          Constants.AMBIENT_NOISE_DEFAULT);
      
    /*  RadioInfo.RadioInfoShared radioInfoSharedWired = RadioInfo.createShared(
              Constants.FREQUENCY_DEFAULT, 
              (int)100e6,
              Constants.TRANSMIT_DEFAULT, 
              Constants.GAIN_DEFAULT,
              Double.MIN_VALUE, 
              Double.MIN_VALUE,
              Constants.TEMPERATURE_DEFAULT, 
              (double)1e-300, 
              Constants.AMBIENT_NOISE_DEFAULT);      */
      // protocol mapper
      Mapper protMap = new Mapper(Constants.NET_PROTOCOL_MAX);
      protMap.mapToNext(Constants.NET_PROTOCOL_DSR);
      protMap.mapToNext(Constants.NET_PROTOCOL_UDP);
      // packet loss
      PacketLoss pl = new PacketLoss.Zero();
      // nodes
      int i=1;
      // radio
      RadioNoise radio1 = new RadioNoiseIndep(i, radioInfoShared);
      RadioNoise radio2 = new RadioNoiseIndep(i+1, radioInfoShared);
      /*RadioNoise radio2w = new RadioNoiseIndep(i+2, radioInfoSharedWired);
      RadioNoise radio3w = new RadioNoiseIndep(i+3, radioInfoSharedWired);*/
      RadioNoise radio3 = new RadioNoiseIndep(i+4, radioInfoShared);
      RadioNoise radio4 = new RadioNoiseIndep(i+5, radioInfoShared);
      
      BusEthernetSwitch bus = new BusEthernetSwitch();
      
      // mac
      Mac802_11 mac1 = new Mac802_11(new MacAddress(i), radio1.getRadioInfo());
      Mac802_11 mac2 = new Mac802_11(new MacAddress(i+1), radio2.getRadioInfo());
      /*MacDumb mac2w = new MacDumb(new MacAddress(i+2), radio2w.getRadioInfo());
      MacDumb mac3w = new MacDumb(new MacAddress(i+3), radio3w.getRadioInfo());  */
      MacDumbBus mac2w = new MacDumbBus(new MacAddress(i+2), (int)100e6);
      MacDumbBus mac3w = new MacDumbBus(new MacAddress(i+3), (int)100e6);        
      Mac802_11 mac3 = new Mac802_11(new MacAddress(i+4), radio3.getRadioInfo()); 
      Mac802_11 mac4 = new Mac802_11(new MacAddress(i+5), radio4.getRadioInfo());
      // net
      NetIp net1 = new NetIp(new NetAddress(i), protMap, pl, pl);
      NetIp net2 = new NetIp(new NetAddress(i+1), protMap, pl, pl);
      NetIp net2w = new NetIp(new NetAddress(i+2), protMap, pl, pl);
      NetIp net3w = new NetIp(new NetAddress(i+3), protMap, pl, pl);
      NetIp net3 = new NetIp(new NetAddress(i+4), protMap, pl, pl);
      NetIp net4 = new NetIp(new NetAddress(i+5), protMap, pl, pl);   
      
      NetMultiInterface  netMI2 = new NetMultiInterface();
      NetMultiInterface  netMI3 = new NetMultiInterface();    
      
      netMI2.addNetIp(net2);
      netMI2.addNetIp(net2w);      
      netMI3.addNetIp(net3);
      netMI3.addNetIp(net3w);
      
      // route
      RouteDsr dsr1 = new RouteDsr(net1.getAddress());
      RouteDsr dsr2 = new RouteDsr(netMI2.getAddress());
      RouteDsr dsr3 = new RouteDsr(netMI3.getAddress());
      RouteDsr dsr4 = new RouteDsr(net4.getAddress());   

      
      // trans
      TransUdp udp1 = new TransUdp();
      //TransUdp udp2 = new TransUdp();
      TransUdp udp4 = new TransUdp();

      // field hookup
      Location location1 = new Location.Location2D(0, 0);
      Location location2 = new Location.Location2D(0, 200);
      Location location3 = new Location.Location2D(0, 2200);
      Location location4 = new Location.Location2D(0, 2400);
      
      field.addRadio(radio1.getRadioInfo(), radio1.getProxy(), location1);
      field.addRadio(radio2.getRadioInfo(), radio2.getProxy(), location2);
      /*fieldWired.addRadio(radio2w.getRadioInfo(), radio2w.getProxy(), location2);
      fieldWired.addRadio(radio3w.getRadioInfo(), radio3w.getProxy(), location3);*/
      bus.addMac(mac2w.getProxy(), new MacAddress(i+2));
      bus.addMac(mac3w.getProxy(), new MacAddress(i+3));     
      field.addRadio(radio3.getRadioInfo(), radio3.getProxy(), location3);   
      field.addRadio(radio4.getRadioInfo(), radio4.getProxy(), location4);      
      // radio hookup
      radio1.setFieldEntity(field.getProxy());
      radio2.setFieldEntity(field.getProxy());
      /*radio2w.setFieldEntity(fieldWired.getProxy());
      radio3w.setFieldEntity(fieldWired.getProxy());*/
      radio3.setFieldEntity(field.getProxy());
      radio4.setFieldEntity(field.getProxy());      

      radio1.setMacEntity(mac1.getProxy());
      radio2.setMacEntity(mac2.getProxy());
      /*radio2w.setMacEntity(mac2w.getProxy());
      radio3w.setMacEntity(mac3w.getProxy());*/
      radio3.setMacEntity(mac3.getProxy());
      radio4.setMacEntity(mac4.getProxy());
      
      // route hookup
      dsr1.setNetEntity(net1.getProxy());
      dsr2.setNetEntity(netMI2.getProxy());
      dsr3.setNetEntity(netMI3.getProxy());
      dsr4.setNetEntity(net4.getProxy());      
      
      // net hookup
      byte intId1 = net1.addInterface(mac1.getProxy());
      byte intId2 = net2.addInterface(mac2.getProxy());
      byte intId2w = net2w.addInterface(mac2w.getProxy());
      byte intId3w = net3w.addInterface(mac3w.getProxy());
      byte intId3 = net3.addInterface(mac3.getProxy());
      byte intId4 = net4.addInterface(mac4.getProxy());
      
      net1.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr2.getProxy());
      net2w.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr2.getProxy());
      net3w.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr3.getProxy());
      net3.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr3.getProxy());
      net4.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr4.getProxy());      
      
      net1.setRouting(dsr1.getProxy());
      net2.setRouting(dsr2.getProxy());
      net2w.setRouting(dsr2.getProxy());
      net3w.setRouting(dsr3.getProxy());
      net3.setRouting(dsr3.getProxy());
      net4.setRouting(dsr4.getProxy());
      
      // mac hookup
      mac1.setRadioEntity(radio1.getProxy());
      mac2.setRadioEntity(radio2.getProxy());
      mac2w.setBusEntity(bus.getProxy());
      mac3w.setBusEntity(bus.getProxy());
      mac3.setRadioEntity(radio3.getProxy());
      mac4.setRadioEntity(radio4.getProxy());
      
      mac1.setNetEntity(net1.getProxy(), intId1);
      mac2.setNetEntity(net2.getProxy(), intId2);
      mac2w.setNetEntity(net2w.getProxy(), intId2w);
      mac3w.setNetEntity(net3w.getProxy(), intId3w);
      mac3.setNetEntity(net3.getProxy(), intId3);
      mac4.setNetEntity(net4.getProxy(), intId4);  
      
      // trans hookup
      udp1.setNetEntity(net1.getProxy());
     // udp2.setNetEntity(net2.getProxy());
      udp4.setNetEntity(net4.getProxy());
      net1.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp1.getProxy());
      //net2.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp2.getProxy());
      net4.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp4.getProxy());

      // applications
      AppJava server = new AppJava(Server.class);
      server.setUdpEntity(udp4.getProxy());
      AppJava client = new AppJava(Client.class);
      client.setUdpEntity(udp1.getProxy());

      // run apps
      server.getProxy().run(null); 
      JistAPI.sleep(1);
      client.getProxy().run(null);
    }
    catch(Exception e) { e.printStackTrace(); }
  }


  public static void main(String args[])
  {
    //TwoNodeTest();
    ThreeNodeTest();
  }
}
