//////////////////////////////////////////////////
// JIST (Java In Simulation Time) Project
// Timestamp: <dsrtest.java Tue 2004/04/06 11:57:46 barr pompom.cs.cornell.edu>
//

// Copyright (C) 2004 by Cornell University
// All rights reserved.
// Refer to LICENSE for terms and conditions of use.

package driver;

import jist.runtime.JistAPI;
import jist.swans.Constants;
import jist.swans.misc.Util;
import jist.swans.misc.Mapper;
import jist.swans.misc.Location;
import jist.swans.field.Field;
import jist.swans.radio.RadioNoise;
import jist.swans.radio.RadioNoiseIndep;
import jist.swans.radio.RadioInfo;
import jist.swans.mac.MacAddress;
import jist.swans.mac.Mac802_11;
import jist.swans.net.NetAddress;
import jist.swans.net.NetIp;
import jist.swans.net.PacketLoss;
import jist.swans.trans.TransUdp;
import jist.swans.app.AppJava;
import jist.swans.route.RouteDsr;
import jist.swans.route.RouteDsrBase.DsrStats;

import java.net.InetAddress;
import java.net.DatagramSocket;
import java.net.DatagramPacket;

public class dsrtest_stat
{

  public static final byte[] SERVER_IP = {1, 2, 3, 4};
  public static final int PORT = 3001;

  public static class Server
  {
    public static void main(String args[])
    {
      try
      {
        System.out.println("server starting at t="+JistAPI.getTime());
        DatagramSocket socket = new DatagramSocket(PORT);
        byte buf[] = new byte[256];
        DatagramPacket packet = new DatagramPacket(buf, buf.length);
        
        for(int i=0;i<100;i++){
	        socket.receive(packet);
	        
	        System.out.println("received at t="+JistAPI.getTime()+
	            " ("+packet.getLength()+" bytes) "
	            +(new String(packet.getData(), packet.getOffset(), packet.getLength())));
        }
        socket.close();
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }

  public static class Client
  {
    public static void main(String args[])
    {
      try
      {
        System.out.println("client starting at t="+JistAPI.getTime());
        DatagramSocket socket = new DatagramSocket();
        String str="";
        

        
        
        
        for (int i=0;i<100;i++){
            str="hi "+i;
            
            byte[] buf = str.getBytes();
        DatagramPacket packet = new DatagramPacket(
          buf,
          buf.length,
          InetAddress.getByAddress(SERVER_IP),
          PORT);
	        System.out.println("sent at t="+JistAPI.getTime());
	        System.out.flush();
	        socket.send(packet);     
	        JistAPI.sleep(Constants.MILLI_SECOND*100);
        }

        
        socket.close();
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }

  private static void TwoNodeTest()
  {
   try
    {
      // field
      Location bounds = new Location.Location2D(200, 200);
      Field field = new Field(bounds);
      /*RadioInfo.RadioInfoShared radioInfoShared = RadioInfo.createShared(
          Constants.FREQUENCY_DEFAULT, 
          Constants.BANDWIDTH_DEFAULT,
          Constants.TRANSMIT_DEFAULT, 
          Constants.GAIN_DEFAULT,
          Util.fromDB(Constants.SENSITIVITY_DEFAULT), 
          Util.fromDB(Constants.THRESHOLD_DEFAULT),
          Constants.TEMPERATURE_DEFAULT, 
          Constants.TEMPERATURE_FACTOR_DEFAULT, 
          Constants.AMBIENT_NOISE_DEFAULT);*/
      
      RadioInfo.RadioInfoShared radioInfoShared = RadioInfo.createShared(
	          Constants.FREQUENCY_DEFAULT, 
	          /*Constants.BANDWIDTH_DEFAULT*/(int)2e6,
	          /*Constants.TRANSMIT_DEFAULT*/24.5, 
	          Constants.GAIN_DEFAULT,
	          /*Util.fromDB(Constants.SENSITIVITY_DEFAULT)*/1.559e-8, 
	          /*Util.fromDB(Constants.THRESHOLD_DEFAULT)*/3.652e-7,
	          Constants.TEMPERATURE_DEFAULT, 
	          Constants.TEMPERATURE_FACTOR_DEFAULT, 
	          Constants.AMBIENT_NOISE_DEFAULT);        
      
      // protocol mapper
      Mapper protMap = new Mapper(Constants.NET_PROTOCOL_MAX);
      protMap.mapToNext(Constants.NET_PROTOCOL_DSR);
      protMap.mapToNext(Constants.NET_PROTOCOL_UDP);
      // packet loss
      PacketLoss pl = new PacketLoss.Zero();
      // nodes
      int i=1;
      // radio
      RadioNoise radio1 = new RadioNoiseIndep(i, radioInfoShared);
      RadioNoise radio2 = new RadioNoiseIndep(i+1, radioInfoShared);
      // mac
      Mac802_11 mac1 = new Mac802_11(new MacAddress(i), radio1.getRadioInfo());
      Mac802_11 mac2 = new Mac802_11(new MacAddress(i+1), radio2.getRadioInfo());
      // net
      NetIp net1 = new NetIp(new NetAddress(SERVER_IP), protMap, pl, pl);
      NetIp net2 = new NetIp(new NetAddress(i), protMap, pl, pl);
      // route
      RouteDsr dsr1 = new RouteDsr(net1.getAddress());
      RouteDsr dsr2 = new RouteDsr(net2.getAddress());
      // trans
      TransUdp udp1 = new TransUdp();
      TransUdp udp2 = new TransUdp();


      // field hookup
      Location location1 = new Location.Location2D(0, 0);
      Location location2 = new Location.Location2D(0, 280);
      field.addRadio(radio1.getRadioInfo(), radio1.getProxy(), location1);
      field.addRadio(radio2.getRadioInfo(), radio2.getProxy(), location2);
      // radio hookup
      radio1.setFieldEntity(field.getProxy());
      radio2.setFieldEntity(field.getProxy());
      radio1.setMacEntity(mac1.getProxy());
      radio2.setMacEntity(mac2.getProxy());
      // route hookup
      dsr1.setNetEntity(net1.getProxy());
      dsr2.setNetEntity(net2.getProxy());
      // net hookup
      byte intId1 = net1.addInterface(mac1.getProxy());
      byte intId2 = net2.addInterface(mac2.getProxy());
      net1.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr2.getProxy());
      net1.setRouting(dsr1.getProxy());
      net2.setRouting(dsr2.getProxy());
      // mac hookup
      mac1.setRadioEntity(radio1.getProxy());
      mac2.setRadioEntity(radio2.getProxy());
      mac1.setNetEntity(net1.getProxy(), intId1);
      mac2.setNetEntity(net2.getProxy(), intId2);
      // trans hookup
      udp1.setNetEntity(net1.getProxy());
      udp2.setNetEntity(net2.getProxy());
      net1.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp2.getProxy());

      // applications
      AppJava server = new AppJava(Server.class);
      server.setUdpEntity(udp1.getProxy());
      AppJava client = new AppJava(Client.class);
      client.setUdpEntity(udp2.getProxy());

      // run apps
      server.getProxy().run(null); 
      JistAPI.sleep(1);
      client.getProxy().run(null);
    }
    catch(Exception e) { e.printStackTrace(); }
  }

  private static void ThreeNodeTest()
  {
   try
    {
      // field
      Location bounds = new Location.Location2D(600, 1500);
      Field field = new Field(bounds);
      RadioInfo.RadioInfoShared radioInfoShared = RadioInfo.createShared(
          Constants.FREQUENCY_DEFAULT, 
          Constants.BANDWIDTH_DEFAULT,
          Constants.TRANSMIT_DEFAULT, 
          Constants.GAIN_DEFAULT,
          Util.fromDB(Constants.SENSITIVITY_DEFAULT), 
          Util.fromDB(Constants.THRESHOLD_DEFAULT),
          Constants.TEMPERATURE_DEFAULT, 
          Constants.TEMPERATURE_FACTOR_DEFAULT, 
          Constants.AMBIENT_NOISE_DEFAULT);
      
    
      
      // protocol mapper
      Mapper protMap = new Mapper(Constants.NET_PROTOCOL_MAX);
      protMap.mapToNext(Constants.NET_PROTOCOL_DSR);
      protMap.mapToNext(Constants.NET_PROTOCOL_UDP);
      // packet loss
      PacketLoss pl = new PacketLoss.Zero();
      // nodes
      int i=1;
      // radio
      RadioNoise radio1 = new RadioNoiseIndep(i, radioInfoShared);
      RadioNoise radio2 = new RadioNoiseIndep(i+1, radioInfoShared);
      RadioNoise radio3 = new RadioNoiseIndep(i+2, radioInfoShared);
      RadioNoise radio4 = new RadioNoiseIndep(i+3, radioInfoShared);
      RadioNoise radio5 = new RadioNoiseIndep(i+4, radioInfoShared);      
      RadioNoise radio6 = new RadioNoiseIndep(i+5, radioInfoShared);      
      
      // mac
      Mac802_11 mac1 = new Mac802_11(new MacAddress(i), radio1.getRadioInfo());
      Mac802_11 mac2 = new Mac802_11(new MacAddress(i+1), radio2.getRadioInfo());
      Mac802_11 mac3 = new Mac802_11(new MacAddress(i+2), radio3.getRadioInfo());
      Mac802_11 mac4 = new Mac802_11(new MacAddress(i+3), radio4.getRadioInfo());
      Mac802_11 mac5 = new Mac802_11(new MacAddress(i+4), radio5.getRadioInfo());
      Mac802_11 mac6 = new Mac802_11(new MacAddress(i+5), radio6.getRadioInfo());     
      
      // net
      NetIp net1 = new NetIp(new NetAddress(i), protMap, pl, pl);
      NetIp net2 = new NetIp(new NetAddress(i+1), protMap, pl, pl);
      NetIp net3 = new NetIp(new NetAddress(SERVER_IP), protMap, pl, pl);
      NetIp net4 = new NetIp(new NetAddress(i+3), protMap, pl, pl);
      NetIp net5 = new NetIp(new NetAddress(i+4), protMap, pl, pl);    
      NetIp net6 = new NetIp(new NetAddress(i+5), protMap, pl, pl);          
      
      // route
      RouteDsr dsr1 = new RouteDsr(net1.getAddress());
      RouteDsr dsr2 = new RouteDsr(net2.getAddress());
      RouteDsr dsr3 = new RouteDsr(net3.getAddress());
      RouteDsr dsr4 = new RouteDsr(net4.getAddress());
      RouteDsr dsr5 = new RouteDsr(net5.getAddress());  
      RouteDsr dsr6 = new RouteDsr(net6.getAddress());       
      
      // trans
      TransUdp udp1 = new TransUdp();
      TransUdp udp2 = new TransUdp();
      TransUdp udp3 = new TransUdp();
      TransUdp udp4 = new TransUdp();
      TransUdp udp5 = new TransUdp();  
      
      
      final DsrStats stats=new DsrStats();      

      // field hookup
      Location location1 = new Location.Location2D(0, 0);
      Location location6 = new Location.Location2D(0, 600);      
      Location location2 = new Location.Location2D(0, 1200);
      Location location3 = new Location.Location2D(0, 1800);
      Location location4 = new Location.Location2D(500, 900);
      Location location5 = new Location.Location2D(500, 1500);     
      
      field.addRadio(radio1.getRadioInfo(), radio1.getProxy(), location1);
      field.addRadio(radio2.getRadioInfo(), radio2.getProxy(), location2);
      field.addRadio(radio3.getRadioInfo(), radio3.getProxy(), location3);
      field.addRadio(radio4.getRadioInfo(), radio4.getProxy(), location4);
      field.addRadio(radio5.getRadioInfo(), radio5.getProxy(), location5);  
      field.addRadio(radio6.getRadioInfo(), radio6.getProxy(), location6);        
      
      // radio hookup
      radio1.setFieldEntity(field.getProxy());
      radio2.setFieldEntity(field.getProxy());
      radio3.setFieldEntity(field.getProxy());
      radio4.setFieldEntity(field.getProxy());
      radio5.setFieldEntity(field.getProxy());   
      radio6.setFieldEntity(field.getProxy());         
      
      radio1.setMacEntity(mac1.getProxy());
      radio2.setMacEntity(mac2.getProxy());
      radio3.setMacEntity(mac3.getProxy());
      radio4.setMacEntity(mac4.getProxy());
      radio5.setMacEntity(mac5.getProxy());     
      radio6.setMacEntity(mac6.getProxy());       
      
      // route hookup
      dsr1.setNetEntity(net1.getProxy());
      dsr2.setNetEntity(net2.getProxy());
      dsr3.setNetEntity(net3.getProxy());
      dsr4.setNetEntity(net4.getProxy());
      dsr5.setNetEntity(net5.getProxy());      
      dsr6.setNetEntity(net6.getProxy());           
      
      dsr1.setStats(stats);
      dsr2.setStats(stats);      
      dsr3.setStats(stats);
      dsr4.setStats(stats);      
      dsr5.setStats(stats);
      dsr6.setStats(stats);      
      
      // net hookup
      byte intId1 = net1.addInterface(mac1.getProxy());
      byte intId2 = net2.addInterface(mac2.getProxy());
      byte intId3 = net3.addInterface(mac3.getProxy());
      byte intId4 = net4.addInterface(mac4.getProxy());
      byte intId5 = net5.addInterface(mac5.getProxy()); 
      byte intId6 = net6.addInterface(mac6.getProxy());      
      
      net1.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr2.getProxy());
      net3.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr3.getProxy());
      net4.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr4.getProxy());
      net5.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr5.getProxy());  
      net6.setProtocolHandler(Constants.NET_PROTOCOL_DSR, dsr6.getProxy());        
      
      net1.setRouting(dsr1.getProxy());
      net2.setRouting(dsr2.getProxy());
      net3.setRouting(dsr3.getProxy());
      net4.setRouting(dsr4.getProxy());
      net5.setRouting(dsr5.getProxy());   
      net6.setRouting(dsr6.getProxy());         
      
      
      // mac hookup
      mac1.setRadioEntity(radio1.getProxy());
      mac2.setRadioEntity(radio2.getProxy());
      mac3.setRadioEntity(radio3.getProxy());
      mac4.setRadioEntity(radio4.getProxy());
      mac5.setRadioEntity(radio5.getProxy()); 
      mac6.setRadioEntity(radio6.getProxy());       
      
      mac1.setNetEntity(net1.getProxy(), intId1);
      mac2.setNetEntity(net2.getProxy(), intId2);
      mac3.setNetEntity(net3.getProxy(), intId3);
      mac4.setNetEntity(net4.getProxy(), intId4);
      mac5.setNetEntity(net5.getProxy(), intId5);  
      mac6.setNetEntity(net6.getProxy(), intId6);        
      
      // trans hookup
      udp1.setNetEntity(net1.getProxy());
      udp2.setNetEntity(net2.getProxy());
      udp3.setNetEntity(net3.getProxy());
      
      
      net1.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp1.getProxy());
      net2.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp2.getProxy());
      net3.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp3.getProxy());

      // applications
      AppJava server = new AppJava(Server.class);
      server.setUdpEntity(udp3.getProxy());
      AppJava client = new AppJava(Client.class);
      client.setUdpEntity(udp1.getProxy());

      // run apps
      server.getProxy().run(null); 
      JistAPI.sleep(1);
      client.getProxy().run(null);
      JistAPI.sleep(Constants.MILLI_SECOND * 500 );
      field.getProxy().delRadio(new Integer(2));
      
      JistAPI.runAt(new Runnable()
      {
        public void run()
        {
          System.out.println(stats);
        }
      }, JistAPI.END);      
      
    }
    catch(Exception e) { e.printStackTrace(); }
  }


  public static void main(String args[])
  {
    //TwoNodeTest();
    ThreeNodeTest();
  }
}
