package driver;


import jargs.gnu.CmdLineParser;
import jist.swans.misc.Location;
import jist.swans.runway.Placement;
import jist.swans.runway.RunWay;

import jist.swans.runway.graph.*;
import edu.uci.ics.jung.graph.filters.*;
import edu.uci.ics.jung.algorithms.connectivity.*;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.utils.GraphUtils;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.*;


import javax.swing.JFrame;
import javax.swing.JPanel;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;


public class highwaycluster {

	static final float PRECISION=0.05f;
	static final int NB_ITER_INIT_RW = 1000;
	static final float TARGET_SPEED = 33.34f;
	
	
	
	  private static class CommandLineOptions
	  {
		  static final int STAT_CLUSTER = 0;
		  static final int STAT_INTER_LINK = 1;
		  
		  int simTime;
		  double dAP;
		  int routeLenght;
		  int intensity;
		  double maxLifeTime;
		  int interCheck;
		  int clusterSizeMax;
		  int stat;
	
		  public String toString(){
			  String s=new String();
			  
			  s+="Temps de simulation : " + simTime + "\n";
			  s+="Densité d'AP : " + dAP + "\n";
			  s+="Taille de l'autoroute : " + routeLenght + "\n";
			  s+="Intensité du réseau de vehicule : " + intensity + "\n";
			  s+="Durée de vie maximum pour un cluster : " + maxLifeTime + "\n";
			  s+="Intervalle entre deux stat : " + interCheck + "\n";	
			  s+="Taille maximum " + clusterSizeMax + "\n";
			  s+="Statistique " + stat + "\n";
			  
			  return s;
		  }
	  }
	  

	
	  private static CommandLineOptions parseCommandLineOptions(String[] args)
	    throws CmdLineParser.OptionException
	  {

	    CmdLineParser parser = new CmdLineParser();
	    CmdLineParser.Option opt_help = parser.addBooleanOption('h', "help");
	    CmdLineParser.Option opt_simTime = parser.addIntegerOption('t', "simTime");
	    CmdLineParser.Option opt_dAP = parser.addDoubleOption('a', "dAP");
	    CmdLineParser.Option opt_routeLenght = parser.addIntegerOption('l', "routeLenght");
	    CmdLineParser.Option opt_intensity = parser.addIntegerOption('i', "intensity");
	    CmdLineParser.Option opt_lifetimemax = parser.addDoubleOption('d', "lifeTimeMax");	    
	    CmdLineParser.Option opt_interCheck = parser.addIntegerOption('I', "interCheck");
	    CmdLineParser.Option opt_clusterSizeMax = parser.addIntegerOption('s', "clusterSizeMax");	    
	    CmdLineParser.Option opt_stat = parser.addIntegerOption('S', "stat");	    
	    
	    CommandLineOptions cmdOpts = new CommandLineOptions();
	    
	    parser.parse(args);
	    
	    if (parser.getOptionValue(opt_simTime) != null){
	    	cmdOpts.simTime = ((Integer)parser.getOptionValue(opt_simTime)).intValue();
	    }else cmdOpts.simTime = 60;
	    if (parser.getOptionValue(opt_dAP) != null){
	    	cmdOpts.dAP = ((Double)parser.getOptionValue(opt_dAP)).doubleValue();
	    }else cmdOpts.dAP = 0;
	    if (parser.getOptionValue(opt_routeLenght) != null){
	    	cmdOpts.routeLenght = ((Integer)parser.getOptionValue(opt_routeLenght)).intValue();
	    }else cmdOpts.routeLenght = 10000;
	    if (parser.getOptionValue(opt_intensity) != null){
	    	cmdOpts.intensity = ((Integer)parser.getOptionValue(opt_intensity)).intValue();
	    }else cmdOpts.intensity = 8;
	    if (parser.getOptionValue(opt_stat) != null){
	    	cmdOpts.stat = ((Integer)parser.getOptionValue(opt_stat)).intValue();
	    }else cmdOpts.stat = 0;	    
	    if (parser.getOptionValue(opt_clusterSizeMax) != null){
	    	cmdOpts.clusterSizeMax = ((Integer)parser.getOptionValue(opt_clusterSizeMax)).intValue();
	    }else cmdOpts.clusterSizeMax = 10;	    
	    
	    if (parser.getOptionValue(opt_lifetimemax) != null){
	    	cmdOpts.maxLifeTime = ((Double)parser.getOptionValue(opt_lifetimemax)).doubleValue();
	    }else cmdOpts.maxLifeTime = 300f;
	    if (parser.getOptionValue(opt_interCheck) != null){
	    	cmdOpts.interCheck = ((Integer)parser.getOptionValue(opt_interCheck)).intValue();
	    }else cmdOpts.interCheck = 10000;
	    
	    
	    return cmdOpts;
	  }
	  




	/*  public static Collection<List<CarVertex>> getConnectedComponent(edu.uci.ics.jung.graph.Graph g,int maxClusterSize){
		  List<List<CarVertex>> components = new LinkedList<List<CarVertex>>();
		  
		  BFSDistanceLabeler bfs = new BFSDistanceLabeler();
		  List<CarVertex> composante;
		  List<CarVertex>  unvisited = new LinkedList<CarVertex> (g.getVertices());
		  
		  while(unvisited.iterator().hasNext()){
				bfs.labelDistances(g,(Vertex)unvisited.iterator().next());
				composante = bfs.getVerticesInOrderVisited();
				
				//System.out.println(composante.size());
				//cut in several component if to big
				//if(composante.size()>maxClusterSize){
					//compute the number of sub component
					int nbsubcpt=(int)Math.ceil((double)composante.size()/(double)maxClusterSize);
					for(int i=0;i<nbsubcpt-1;i++){
						//System.out.println(i*maxClusterSize+"->"+(i+1)*maxClusterSize);
						components.add(composante.subList(i*maxClusterSize, (i+1)*maxClusterSize));					
					}
					//System.out.println((nbsubcpt-1)*maxClusterSize+"->"+composante.size());
					
					components.add(composante.subList((nbsubcpt-1)*maxClusterSize, composante.size()));
					

				
				
				
				unvisited.removeAll(  composante);	
				
		  }
		  
		  return components;
	  }
	*/

	public static void viewGraph(edu.uci.ics.jung.graph.Graph g,String title){
		final VisualizationViewer vv = new VisualizationViewer(new FRLayout(g),
		        new PluggableRenderer(),new Dimension(1250,950));
		
		final DefaultModalGraphMouse graphMouse = new DefaultModalGraphMouse();
		vv.setGraphMouse(graphMouse);

		
		//final ScalingControl scaler = new CrossoverScalingControl();	     
		
        JPanel jp = new JPanel();
        jp.setBackground(Color.WHITE);
        jp.setLayout(new BorderLayout());
        jp.add(vv, BorderLayout.CENTER);			

        JFrame jf = new JFrame();
        jf.setTitle(title);
        jf.getContentPane().add(jp);

        jf.pack();
        jf.setVisible(true);
	}
	
	public static void processStat(float t,Set<Cluster> clusters){

		  for(Iterator<Cluster>  i=clusters.iterator();i.hasNext();){
			  Cluster cluster=i.next();
			  if (cluster.size()>1){
				  	cluster.processStat(t);
			  }
		  }		
		  //System.out.println("Nombre de vehicule dans les clusters : "+sizetotal);
	}
	
	public static Set<Edge> getInterLink(edu.uci.ics.jung.graph.Graph g,Set<Cluster> clusters){
		Set<Edge> links=new HashSet(g.getEdges());
		
		  for(Iterator<Cluster>  i=clusters.iterator();i.hasNext();){
			  Cluster c = i.next();
			  links.removeAll((Collection)  c.getEdgesInCluster() );
			  //System.out.println(c);
		  }				
		
		return links;
	}
	
	public static void showStatInterLink(CommandLineOptions options,Set<Cluster> clusters,edu.uci.ics.jung.graph.Graph g){
		Set<Edge> links=getInterLink(g,clusters);
		//System.out.println(links);		
		//System.out.println(links.size());
		int nbLinkStable=0;
		int nbInterLinkStable=0;		
		
		for(Iterator<Edge>  i=links.iterator();i.hasNext();){
			
			if (EdgeStat.lifeTime( i.next()) > options.maxLifeTime)
				nbInterLinkStable++;
		
		}
		
		for(Iterator<Edge>  i=g.getEdges().iterator();i.hasNext();){
			
			if (EdgeStat.lifeTime( i.next()) > options.maxLifeTime)
				nbLinkStable++;
		
		}		
		
		System.out.println(nbInterLinkStable + " " +  links.size() + " " + nbLinkStable + " " +g.numEdges());
	}
	
	public static void showStat(CommandLineOptions options,Set<Cluster> clusters){
		  int n=0;
		  int sizetotal=0;
		  for(Iterator<Cluster>  i=clusters.iterator();i.hasNext();){
			  Cluster cluster=i.next();
			  sizetotal+=cluster.size();
			  if (cluster.size()>1){
				  n++;
				  //System.out.println(cluster);

				  
				  //System.out.println(cluster.getEdgesInCluster());
				  

				  
				  //temps id nb_vehicule longueur life_time fragilité   
				  System.out.println(options.intensity+" "+options.clusterSizeMax+" "+options.maxLifeTime+" "+ n+" "+cluster.stat());
				  //System.out.println();
			  }
		  }		
		  //System.out.println("Nombre de vehicule dans les clusters : "+sizetotal);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws CmdLineParser.OptionException {
		// TODO Auto-generated method stub

		  CommandLineOptions options;
		  
		  options = parseCommandLineOptions(args);
		  
		  System.err.println(options);
		  
		  
		  RunWay rw=new RunWay(3,3,options.routeLenght,TARGET_SPEED,options.intensity,PRECISION,true,new Location.Location2D(1,4));
		  Placement p = new Placement(rw);
		  rw.setPlacement(p);
		  rw.next();
		  rw.setCirculaire(true);
		  
		  System.err.println("Creation du graphe ");
		  
		  
		  Graph gutil=new Graph(rw,0.0, 0 );
		  edu.uci.ics.jung.graph.Graph g = gutil.getGraphe();
		  
		  System.err.println("\tNombre de noeud : " + g.numVertices());
		  System.err.println("\tNombre d'arete : " + g.numEdges());		  
		
		  System.err.println("Initialisation de la route");
		  for(int i=0;i<NB_ITER_INIT_RW;i++)
			  rw.next();
		  
		  System.err.println("Initialisation du graphe");
		  gutil.updateGraphe();
		  
		  //viewGraph(g,"Initial graph");
		  
		  LifeTimeFilter f = new LifeTimeFilter((float)options.maxLifeTime);
		  /* edu.uci.ics.jung.graph.Graph sg = f.filter(g).assemble();*/
		  
		  //viewGraph(sg,"Filtered graph");
		  
		 /* System.out.println("Taille du graphe initial : "+ g.getEdges().size());
		  System.out.println("Taille du graphe filtré : "+  sg.getEdges().size());*/
		  

		 
		 
		  //Collection<List<CarVertex>> components = getConnectedComponent(sg,options.clusterSizeMax);
		  
		  Clusterise clusterisation = new Clusterise(gutil.getGraphe());
		  clusterisation.setFilter(f);
		  clusterisation.setSizeMax(options.clusterSizeMax);
		  clusterisation.clusteriser();
		  //showStat(0,options, clusterisation.getClusters());
		  
		  int nb_iter = (int)((float)options.simTime/PRECISION);
		  int intercheck = (int)((float)options.interCheck/(PRECISION*1000f));
		  
		  showStatInterLink(options, clusterisation.getClusters(),gutil.getGraphe());
		  
		  for(int i=0;i<nb_iter;i++){
			  //edu.uci.ics.jung.graph.Graph g = gutil.getGraphe();
			  //System.out.println(g);

			  //System.out.println(gutil.getShortestPath(5, 50));
			  
			  
			  
			  
			  if(i%intercheck == 0){
				  System.err.print("Simulation "+((((double)i)/(double)nb_iter)*100.0f)+" %                                                \r");
				  gutil.updateGraphe();
				  
				  if(options.stat==0)
					  processStat((float)i*PRECISION, clusterisation.getClusters());
				  
				  showStatInterLink(options, clusterisation.getClusters(),gutil.getGraphe());
				  
				  //showStatInterLink(options, clusterisation.getClusters(),gutil.getGraphe());
			  }
			  
			  rw.next();

		  }
		  
		  if(options.stat==0)
			  showStat(options, clusterisation.getClusters());
		  else if(options.stat==1)
			  showStatInterLink(options, clusterisation.getClusters(),gutil.getGraphe());
		  


		 //System.out.println(options.maxLifeTime + " "+ options.intensity + " " + getConnectedComponent(sg).size());
		 
		 
		 
		  
	}

}
