package driver;

import jist.runtime.JistAPI;
import jist.swans.Constants;
import jist.swans.bus.BusEthernetSwitch;
import jist.swans.field.Fading;
import jist.swans.field.Mobility;
import jist.swans.field.PathLoss;
import jist.swans.field.Placement;
import jist.swans.field.Spatial;
import jist.swans.mac.Mac802_11;
import jist.swans.mac.MacAddress;
import jist.swans.mac.MacDumbBus;
import jist.swans.misc.Location;
import jist.swans.misc.Mapper;
import jist.swans.misc.Util;
import jist.swans.net.NetAddress;
import jist.swans.net.NetIp;
import jist.swans.net.NetMultiInterface;
import jist.swans.net.PacketLoss;
import jist.swans.field.Field;
import jist.swans.radio.RadioInfo;
import jist.swans.radio.RadioNoise;
import jist.swans.radio.RadioNoiseIndep;
import jist.swans.route.RouteDsr;
import jist.swans.route.RouteAodv;
import jist.swans.route.RouteInterface;
import jist.swans.route.RouteDsrBase.DsrStats;
import jist.swans.runway.RunWay;
import jist.swans.runway.RunWayCar;
import jist.swans.runway.RunWayCar.RunWayCarStats;
import jist.swans.runway.NewCarEventInterface;
import jist.swans.runway.JRunWayWindow;
import jist.swans.trans.CircularBuffer;
import jist.swans.trans.TransUdp;
import jist.runtime.JistAPI;
import jist.swans.app.AppJava;
import jargs.gnu.CmdLineParser;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.Hashtable;
import java.util.Random;
import java.util.ArrayList;
import java.util.Iterator;

import jist.swans.field.Field.RadioData;

public class hybridrunway implements NewCarEventInterface{
	
	
	  private static class TraficSource
	  {
		  int idClient;
		  int idServer;
		  MyCBR.Client client;
		  MyCBR.Server server;
	  }
	  
	  private static class OptionsRadio
	  {

		    /**
		     * Bandwidth (units: bits/second).
		     */
		    protected int bandwidth;

		    /**
		     * Transmission power (units: dBm).
		     */
		    protected double transmit;

		    /**
		     * Reception sensitivity (units: mW).
		     */
		    protected double sensitivity_mW;

		    /**
		     * Reception threshold (units: mW).
		     */
		    protected double threshold_mW;
  
		    public String toString(){
		    	String s="";
		    	
		    	s+="Bande passante : "+bandwidth+" bits/s ";
		    	s+="Puissance : "+transmit+" dBm ";
		    	s+="Sensibilité : " + sensitivity_mW+" mW ";
		    	s+="Seuil : " + threshold_mW+" mW";		    	
		    	
		    	return s;
		    }
	  }
	  
	  private static class CommandLineOptions
	  {
		  boolean help;
		  int nbrLane;
		  float sizeLan;
		  float sizeRunway;
		  float speedTarget;
		  float intensity;
		  float precision;
		  boolean doubleMind;
		  boolean trafficstatic;
		  boolean circulaire;
		  int nbrAP;
		  int protocol;
		  boolean verbose;
		  int nbSource;
		  float bitrate;
		  int sizePacket;
		  int timeSim;
		  boolean gui;
		  OptionsRadio optRadio;
		  //ajouté temps de simu
		  
		  public String toString(){
			  String s="";
			  
			  s+="Nombre de voie :"+nbrLane+"\n";
			  s+="Taille d'une voie : "+sizeLan+"\n";
			  s+="Longueur de l'autoroute : "+sizeRunway+"\n";
			  s+="Vitesse cible moyenne : "+speedTarget+"\n";
			  s+="Intensité : "+intensity+"\n";
			  s+="Precision : "+precision+"\n";
			  s+="Double sens : "+doubleMind+"\n";
			  s+="Traffic statique : "+trafficstatic+"\n";			  
			  s+="Nombre de point d'accès : "+nbrAP+"\n";
			  s+="Protocole : "+protocol+"\n";
			  s+="Débit : "+bitrate+"\n";	
			  s+="Taille d'un paquet : "+sizePacket+"\n";
			  s+="Radio : "+optRadio+"\n";
			  s+="Temps du simulation : "+timeSim+"\n";
			  
			  return s;
		  }
	  }
	  
	  /** Prints a usage statement. */
	  private static void showUsage() 
	  {
	    System.out.println("Usage: jist jist.swans.Main  driver.hybridrunway [options]");
	    System.out.println();
	    System.out.println("  -h, --help           affiche ce message");
	    System.out.println("  -n, --nbrlane        nombre de voie");
	    System.out.println("  -T, --timesim        temps de la simulation");	    
	    System.out.println("  -s, --sizelane       taill d'une voie (en m)");
	    System.out.println("  -l, --sizerunway     longueur de l'autoroute (en m)");
	    System.out.println("  -t, --speedtarget    vitesse cible moyenne (en m/s)");
	    System.out.println("  -i, --intensity      intensité (en nombre de véhicule par kilomettre)");
	    System.out.println("  -d, --precision      precision (en seconde). Temps d'attende entre chaque étape");
	    System.out.println("  -m, --doublemind     double sens");
	    System.out.println("  -S, --static     	   les véhicules ne bouge pas");
	    System.out.println("  -c, --circulaire     autoroute circulaire (rond point géant)");	    
	    System.out.println("  -a, --nopmbreap      nombre de point d'accéss");
	    System.out.println("  -p, --protocole      protocole (DSR,AODV ou ZRP)");
	    System.out.println("  -b, --nbsource       nombre de source");	    
	    System.out.println("  -r, --bitrate        Débit (en bits/s");
	    System.out.println("  -k, --sizepacket     Taille d'un paquet (en octet");
	    System.out.println("  -R, --radioinfo      Proprièté de la transmission radio (bande passante:puissance:sensibilite:seuil)");	    
	    System.out.println("  -v, --verbose        Affiche toutes les statistiques");	    
	    System.out.println("  -g, --gui            Affiche une GUI");	    

	    System.out.println();
	    System.out.println("Sortie par défaut : delai moyen,delai max,PDR,Overhead,longeure de chemin moyen");	    
	    System.out.println();
	    System.out.println("e.g.");
	    System.out.println("  jist jist.swans.Main ");
	    System.out.println();
	  }
  
	  
	  private static CommandLineOptions parseCommandLineOptions(String[] args)
	    throws CmdLineParser.OptionException
	  {

	    CmdLineParser parser = new CmdLineParser();
	    CmdLineParser.Option opt_help = parser.addBooleanOption('h', "help");
	    CmdLineParser.Option opt_nbrLane = parser.addIntegerOption('n', "nbrlane");
	    CmdLineParser.Option opt_timesimu = parser.addIntegerOption('T', "timesimu");	    
	    CmdLineParser.Option opt_sizelane = parser.addDoubleOption('s', "sizelane");
	    CmdLineParser.Option opt_sizerunway = parser.addDoubleOption('l', "sizerunway");
	    CmdLineParser.Option opt_speedtarget = parser.addDoubleOption('t', "speedtarget");
	    CmdLineParser.Option opt_intensity = parser.addDoubleOption('i', "intensity");
	    CmdLineParser.Option opt_presicion = parser.addDoubleOption('d', "precision");
	    CmdLineParser.Option opt_doublemind = parser.addBooleanOption('m', "doublemind");
	    CmdLineParser.Option opt_static = parser.addBooleanOption('S', "static");
	    CmdLineParser.Option opt_circ = parser.addBooleanOption('c', "circulaire");	    
	    CmdLineParser.Option opt_nombreap = parser.addIntegerOption('a', "nombreap");
	    CmdLineParser.Option opt_protocol = parser.addStringOption('p', "protocol");
	    CmdLineParser.Option opt_verbose = parser.addBooleanOption('v', "verbose");
	    CmdLineParser.Option opt_bitrate = parser.addDoubleOption('r', "bitrate");
	    CmdLineParser.Option opt_nbsource = parser.addIntegerOption('b', "nbsource");	
	    CmdLineParser.Option opt_radioinfo = parser.addStringOption('R', "radioinfo");		    
	    CmdLineParser.Option opt_sizepacket = parser.addIntegerOption('k', "sizepacket");
	    
	    CmdLineParser.Option opt_gui = parser.addBooleanOption('g', "gui");
	    
	    parser.parse(args);
	    
		  
		    CommandLineOptions cmdOpts = new CommandLineOptions();
		    cmdOpts.optRadio=new OptionsRadio();
		    // help
		    if(parser.getOptionValue(opt_help) != null)
		    {
		      cmdOpts.help = true;
		    }
		    
		    if(parser.getOptionValue(opt_doublemind) != null)
		    {
		      cmdOpts.doubleMind = true;
		    }
		    	
		    
		    if(parser.getOptionValue(opt_static) != null)
		    {
		      cmdOpts.trafficstatic = true;
		    }		
		    
		    if(parser.getOptionValue(opt_circ) != null)
		    {
		      cmdOpts.circulaire = true;
		    }				    
		    
		    if(parser.getOptionValue(opt_verbose) != null)
		    {
		      cmdOpts.verbose = true;
		    }	
		    
		    if(parser.getOptionValue(opt_gui) != null)
		    {
		      cmdOpts.gui = true;
		    }	
		    
		    if(parser.getOptionValue(opt_timesimu) != null)
		    {
		    	cmdOpts.timeSim= ((Integer)parser.getOptionValue(opt_timesimu)).intValue();
		    }else
		    	cmdOpts.timeSim=15;
		    
		    if(parser.getOptionValue(opt_nbrLane) != null)
		    {
		    	cmdOpts.nbrLane= ((Integer)parser.getOptionValue(opt_nbrLane)).intValue();
		    }else
		    	cmdOpts.nbrLane=3;
		    
		    if(parser.getOptionValue(opt_nombreap) != null)
		    {
		    	cmdOpts.nbrAP= ((Integer)parser.getOptionValue(opt_nombreap)).intValue();
		    	cmdOpts.nbrAP=Math.min(cmdOpts.nbrAP,50);
		    }else
		    	cmdOpts.nbrAP = 0;
	
		    if(parser.getOptionValue(opt_sizepacket) != null)
		    {
		    	cmdOpts.sizePacket= ((Integer)parser.getOptionValue(opt_sizepacket)).intValue();
		    }else
		    	cmdOpts.sizePacket = 128;	
		    
		    if(parser.getOptionValue(opt_nbsource) != null)
		    {
		    	cmdOpts.nbSource= ((Integer)parser.getOptionValue(opt_nbsource)).intValue();
		    }else
		    	cmdOpts.nbSource = 1;			    
		    
		    if(parser.getOptionValue(opt_protocol) != null)
		    {
		    	String protString = (String)parser.getOptionValue(opt_protocol);
		    	
		    	if(protString.equalsIgnoreCase("DSR"))
		    		cmdOpts.protocol=Constants.NET_PROTOCOL_DSR;
		    	else if(protString.equalsIgnoreCase("AODV"))
		    		cmdOpts.protocol=Constants.NET_PROTOCOL_AODV;
		    	else if(protString.equalsIgnoreCase("ZRP"))
		    		cmdOpts.protocol=Constants.NET_PROTOCOL_ZRP;		    		
		    }else	
		    	cmdOpts.protocol=Constants.NET_PROTOCOL_DSR;
		    
		    if(parser.getOptionValue(opt_sizerunway) != null)
		    {
		    	cmdOpts.sizeRunway= ((Double)parser.getOptionValue(opt_sizerunway)).floatValue();
		    }else
		    	cmdOpts.sizeRunway=5000f;
		    
		    if(parser.getOptionValue(opt_speedtarget) != null)
		    {
		    	cmdOpts.speedTarget= ((Double)parser.getOptionValue(opt_speedtarget)).floatValue();
		    }else
		    	cmdOpts.speedTarget=33.34f;
		    
		    if(parser.getOptionValue(opt_bitrate) != null)
		    {
		    	cmdOpts.bitrate= ((Double)parser.getOptionValue(opt_bitrate)).floatValue();
		    }else
		    	cmdOpts.bitrate=10240;		    
		    
		    if(parser.getOptionValue(opt_intensity) != null)
		    {
		    	cmdOpts.intensity= ((Double)parser.getOptionValue(opt_intensity)).floatValue();
		    }else
		    	cmdOpts.intensity=5;
		    
		    if(parser.getOptionValue(opt_sizelane) != null)
		    {
		    	cmdOpts.sizeLan= ((Double)parser.getOptionValue(opt_sizelane)).floatValue();
		    }else
		    	cmdOpts.sizeLan=3;		    	
		    
		    if(parser.getOptionValue(opt_presicion) != null)
		    {
		    	cmdOpts.precision= ((Double)parser.getOptionValue(opt_presicion)).floatValue();
		    }else
		    	cmdOpts.precision=0.05f;
		    
		    
		    if(parser.getOptionValue(opt_radioinfo) != null)
		    {
		    	String opt[] =  ((String)parser.getOptionValue(opt_radioinfo)).split(":") ;
		    	
		    	
		    	
		    	cmdOpts.optRadio.bandwidth= Integer.parseInt(opt[0])*((int)1e6);
		    	cmdOpts.optRadio.transmit=Double.parseDouble(opt[1]);
		    	cmdOpts.optRadio.sensitivity_mW=Double.parseDouble(opt[2]);
		    	cmdOpts.optRadio.threshold_mW=Double.parseDouble(opt[3]);
		    	
		    }else{
		    	cmdOpts.optRadio.bandwidth=(int)2e6;
		    	cmdOpts.optRadio.sensitivity_mW=1.559e-8;
		    	cmdOpts.optRadio.threshold_mW=3.652e-7;
		    	cmdOpts.optRadio.transmit=24.5;
		    	
		    }
		    		    
		    
		    return cmdOpts;
	  }
	  
		public  DsrStats statsDsr=new DsrStats();
		public  RouteAodv.AodvStats statsAodv=new RouteAodv.AodvStats();		
		public  RunWayCarStats statsRw=new RunWayCarStats() ;
		protected NewCarEventInterface self;
		public  RunWay[] rw;
		public  Location bounds;
		public  Field field;
		public  RadioInfo.RadioInfoShared radioInfoShared;
		public  Mapper protMap;
		public  PacketLoss pl;
		public  BusEthernetSwitch etherSwitch;	
		public jist.swans.runway.Placement[] place;
		public int nextIdNode=0;
		public int protocole=0;
		public CommandLineOptions options;
		public ArrayList traficSources;
		
		public void deleteCarEvent(int id){
		}
	public void newCarEvent(int id,Location loc){
		//System.out.println("new car !! " +nextIdNode + " "+ loc + JistAPI.getTime());
		
	    addMobileNode( id,loc);

		
	}
	
	public hybridrunway() {
		
		this.self = (NewCarEventInterface)JistAPI.proxy(this, NewCarEventInterface.class);
	}	
	
	public boolean addMobileNode(int i,Location location){
		

		NetAddress address=new NetAddress(i);
		
		RadioNoise radio = new RadioNoiseIndep(i, radioInfoShared);
		Mac802_11 mac = new Mac802_11(new MacAddress(i), radio.getRadioInfo());
		NetIp net = new NetIp(address, protMap, pl, pl);
		
		
		
		
		TransUdp udp = new TransUdp();

		
	    (field.getProxy()).addRadio(radio.getRadioInfo(), radio.getProxy(), location);
	    
	    //System.out.println("loc "+i+" "+location);
	    
	    //rw.init(radio.getRadioInfo().getUnique().getID(),location,carStats);
	    //field.startMobility(radio.getRadioInfo().getUnique().getID());
	    //field.initMobility(radio.getRadioInfo().getUnique().getID());
	    
	    radio.setFieldEntity(field.getProxy());
	    radio.setMacEntity(mac.getProxy());
	    
	    
	      // routing
	      RouteInterface route = null;
	      switch(protocole)
	      {
	        case Constants.NET_PROTOCOL_DSR:
	          RouteDsr dsr = new RouteDsr(net.getAddress());
	          route = dsr.getProxy();
	          dsr.setNetEntity(net.getProxy());
	          dsr.setStats(statsDsr);
	          break;
	        case Constants.NET_PROTOCOL_AODV:
	          RouteAodv aodv = new RouteAodv(net.getAddress());
	          route = aodv.getProxy();
	          aodv.setNetEntity(net.getProxy());
	          aodv.setStats(statsAodv);
	          aodv.getProxy().start();
	          break;
	      }
	    
	    
	    
	    byte intId = net.addInterface(mac.getProxy());
	    net.setProtocolHandler(protocole, route);
	    net.setRouting(route);
	    mac.setRadioEntity(radio.getProxy());
	    mac.setNetEntity(net.getProxy(), intId);
	    udp.setNetEntity(net.getProxy());
	    net.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp.getProxy());

	    //udpEntity.put(radio.getRadioInfo().getUnique().getID(), udp);
	    

	    
	  for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
		  TraficSource t=(TraficSource)it.next();
		  if(t.idServer==i){
			  t.server = new MyCBR.Server(udp.getProxy(),address);
			  t.server.getProxy().run();
			  if(options.verbose)
				  System.out.println("ajout du serveur "+address+" "+location);
		  }
	
		  if(t.idClient==i){
			  t.client = new MyCBR.Client(udp.getProxy(),options.bitrate,options.sizePacket,options.timeSim,address,new NetAddress(t.idServer));
			  t.client.getProxy().run();
			  if(options.verbose)
				  System.out.println("ajout client cbr "+address+" "+location);
		  }
		  
	  }	    
	    

	    
	    //System.out.println(location);
	    
	    return true;
	}
	
	public void addGatewayNode(int i,int iw,Location location){
		
		RadioNoise radio = new RadioNoiseIndep(i, radioInfoShared);
		Mac802_11 mac = new Mac802_11(new MacAddress(i), radio.getRadioInfo());
		MacAddress macAddrW=new MacAddress(iw);
		MacDumbBus macw = new MacDumbBus(macAddrW, (int)100e6);
		NetIp net = new NetIp(new NetAddress(i), protMap, pl, pl);
		NetIp netw = new NetIp(new NetAddress(iw), protMap, pl, pl);
	    NetMultiInterface  netMI = new NetMultiInterface();
	    
	    netMI.addNetIp(net);
	    netMI.addNetIp(netw);   
	   
	    
	    
		  // routing
		  RouteInterface route = null;
		  switch(protocole)
		  {
		    case Constants.NET_PROTOCOL_DSR:
		      RouteDsr dsr = new RouteDsr(net.getAddress());
		      route = dsr.getProxy();
		      dsr.setNetEntity(netMI.getProxy());
		      dsr.setStats(statsDsr);
		      break;
		    case Constants.NET_PROTOCOL_AODV:
		      RouteAodv aodv = new RouteAodv(net.getAddress());
		      route = aodv.getProxy();
		      aodv.setNetEntity(netMI.getProxy());
		      aodv.getProxy().start();
		      aodv.setStats(statsAodv);
		      break;
		  }
	    	    
	    
	    
	    field.addRadio(radio.getRadioInfo(), radio.getProxy(), location);
	    etherSwitch.addMac(macw.getProxy(), macAddrW);
	    radio.setFieldEntity(field.getProxy());
	    radio.setMacEntity(mac.getProxy());

	    
	    byte intId = net.addInterface(mac.getProxy());
	    byte intIdw = netw.addInterface(macw.getProxy());	 
	    
	    mac.setNetEntity(net.getProxy(), intId);
	    macw.setNetEntity(netw.getProxy(), intIdw);
	    
	    net.setProtocolHandler(protocole, route);
	    netw.setProtocolHandler(protocole, route);	 
	    
	    net.setRouting(route);
	    netw.setRouting(route);
	    
	    mac.setRadioEntity(radio.getProxy());
	    macw.setBusEntity(etherSwitch.getProxy());
	    //System.out.println(location);
	}
	




	private double getStatDelayAvg(){
		double val=0.0;
	    
		for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
			TraficSource t=(TraficSource)it.next();
			val+=t.server.getDelayAvg();
		}
		
		return val/(double)traficSources.size();
	}
	
	private double getStatDelayMax(){
		double val=0.0;
	    
		for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
			TraficSource t=(TraficSource)it.next();
			if(val<t.server.getDelayMax())
				val=t.server.getDelayMax();
		}
		
		return val;		
	}	

	private double getStatThroughput(){
		double val=0.0;
	    
		for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
			TraficSource t=(TraficSource)it.next();
			
			val+= ((double)(t.server.getPacketsSizeReceived()*8))  /  (double)(options.timeSim)  ;

		}
		
		return val/(double)traficSources.size();		
	}
	
	private double getStatPDR(){
		double val=0.0;
	    
		for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
			TraficSource t=(TraficSource)it.next();
			val+=((double)t.server.getPacketsReceived()/(double)t.client.getPacketsSended());
		}
		
		return val/(double)traficSources.size();			
	}
	
	private double getNbPacketSend(){
		int val=0;
	    
		for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
			TraficSource t=(TraficSource)it.next();
			val+=t.client.getPacketsSended();
		}
		
		return val;			
	}	
	
	private double getNbPacketReceived(){
		int val=0;
	    
		for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
			TraficSource t=(TraficSource)it.next();
			val+=t.server.getPacketsReceived();
		}
		
		return val;			
	}		

	public void  buildFieldRunway(){


	    if(options.doubleMind)
	    	rw= new RunWay[2];
	    else
	    	rw= new RunWay[1];	    	

		float sizeRw=options.sizeRunway;
	    float ray=(sizeRw/(float)(2.0*Math.PI));
		
		rw[0]=new RunWay(options.nbrLane,options.sizeLan,sizeRw,options.speedTarget,options.intensity,options.precision,options.doubleMind,new Location.Location2D(1,4));
		
		if(options.doubleMind)
			rw[1]=new RunWay(options.nbrLane,options.sizeLan,sizeRw,options.speedTarget,options.intensity,options.precision,!options.doubleMind,new Location.Location2D(1,14));
		
		
		//rw[2]=new RunWay(2,3,5000,33.34f,10,0.050f,true,new Location.Location2D(1,30));
		//rw[3]=new RunWay(2,3,5000,33.34f,10,0.050f,false,new Location.Location2D(1,35));		
		
		
			
		statsDsr.clear();
		statsAodv.clear();
		
		// field
		if(options.circulaire)
		  bounds = new Location.Location2D(2*ray+10, 2*ray+10);			
		else
	      bounds = new Location.Location2D(sizeRw+1, 100);
		
		if(options.verbose)
			System.out.println("Bounds : "+bounds);
	      
	      field = new Field(bounds);	
		
		place= new jist.swans.runway.Placement[rw.length];
		for (int j=0;j<rw.length;j++){
		 place[j]=new jist.swans.runway.Placement(rw[j]);
		 rw[j].setCarsStats(statsRw);
		 rw[j].setNewCarEvent(self);
		 rw[j].setPlacement(place[j]);
		 rw[j].setField(field.getProxy());
	     rw[j].setNextId(100+j*1000);
	     rw[j].setCirculaire(options.circulaire);
		 statsRw.clear(rw[j]);
		 
		}
			 
				
		

	      //field.setMobility(rw[0]);
	      
	      radioInfoShared = RadioInfo.createShared(
	          Constants.FREQUENCY_DEFAULT, 
	          /*Constants.BANDWIDTH_DEFAULT*/options.optRadio.bandwidth,
	          /*Constants.TRANSMIT_DEFAULT*/options.optRadio.transmit, 
	          Constants.GAIN_DEFAULT,
	          /*Util.fromDB(Constants.SENSITIVITY_DEFAULT)*/options.optRadio.sensitivity_mW, 
	          /*Util.fromDB(Constants.THRESHOLD_DEFAULT)*/options.optRadio.threshold_mW,
	          Constants.TEMPERATURE_DEFAULT, 
	          Constants.TEMPERATURE_FACTOR_DEFAULT, 
	          Constants.AMBIENT_NOISE_DEFAULT);
	      
	      // protocol mapper
	      protMap = new Mapper(Constants.NET_PROTOCOL_MAX);
	      protMap.mapToNext(options.protocol);
	      protMap.mapToNext(Constants.NET_PROTOCOL_UDP);
	      // packet loss
	      pl = new PacketLoss.Zero();		
	      
	      protocole=options.protocol;

	      
	      //udpEntity = new Hashtable();
	      
	      etherSwitch = new BusEthernetSwitch();
	      
	      float inter=sizeRw/(options.nbrAP+1);

	      
	      for(int i=1;i<=options.nbrAP;i++){
	    	  float x,y;
	    	  if(options.circulaire){
	    		  float rad=((float)i/(float)(options.nbrAP))*2.0f*(float)Math.PI;
		  		  x = (ray-1.0f)*(float)Math.cos(rad) +ray ;
				  y = (ray-1.0f)*(float)Math.sin(rad) +ray ;
	    	  }else{
	    		  x=inter*i;
	    		  y=1;
	    	  }
	    	  
		      addGatewayNode(nextIdNode++,nextIdNode++,new Location.Location2D(x,y));
	      }
	      

	      
          if(options.verbose){	    
		      JistAPI.runAt(new Runnable()
		      {
		        public void run()
		        {
		        	
		
			        	
			          System.out.println(statsRw);
			          
			          switch(protocole){
				          case Constants.NET_PROTOCOL_DSR:
					          System.out.println(statsDsr);
					          System.out.println("Overhead : "+(double)statsDsr.send.dsrSizePackets/(statsDsr.send.dsrSizePackets+statsDsr.send.dataSizePackets));
					          break;
				          
				          case Constants.NET_PROTOCOL_AODV:
					          System.out.println(statsAodv);
					          //System.out.println("Overhead : "+(double)statsDsr.send.dsrSizePackets/(statsDsr.send.dsrSizePackets+statsDsr.send.dataSizePackets));
					          break;				          
			          }
			          System.out.println("Delai moyen : "+getStatDelayAvg()+"ms");
			          System.out.println("Delai max : "+getStatDelayMax()+"ms");	          
			          System.out.println("PDR : "+getStatPDR() + " " +getNbPacketReceived() +  "/" +  getNbPacketSend());
			          System.out.println("Throughput : "+getStatThroughput()+"bits/s");			          
			          
		        }
		      }, JistAPI.END);   
	      
          }else{
		      JistAPI.runAt(new Runnable()
		      {
		        public void run()
		        {        	  
		        	switch(protocole){
			          case Constants.NET_PROTOCOL_DSR:
			        	  System.out.println(getStatDelayAvg()+" "+getStatDelayMax()+" "+getStatPDR() + " " + getStatThroughput() + " " +(double)statsDsr.send.dsrSizePackets/(statsDsr.send.dsrSizePackets+statsDsr.send.dataSizePackets) + " " + statsDsr.avgLength());
			        	  break;
			          case Constants.NET_PROTOCOL_AODV:
			        	  System.out.println(getStatDelayAvg()+" "+getStatDelayMax()+" "+getStatPDR() + " " + getStatThroughput() + " 0.0"  + " 0.0");
			        	  break;		        	
		        	}
		        }
    		}, JistAPI.END);          	  
          }
          
          JistAPI.endAt((options.timeSim+30)*Constants.SECOND);
	        	  

          int nbstep=(int)(((float)(options.timeSim+5))/options.precision);
          
          if(options.trafficstatic)
        	  nbstep=1;
	      //System.out.println(id-1);
          JRunWayWindow rwWin=null;
          
          if(options.gui)
        	  rwWin=new JRunWayWindow(rw,30.0f,3000.0f);
	      
	      for(int i=0;i<nbstep;i++){
	    	  
	    	    for (int j=0;j<rw.length;j++)
	    	    	rw[j].next();
	    	  	
	    	    if(rwWin!=null){
	    	    	rwWin.repaint();
	    	    }
	    	  	
	    	  	if(i==0){

	    	          Random rnd=new Random();
	    	          traficSources=new ArrayList(options.nbSource);
	    	          for(int j=0;j<options.nbSource;j++){
	    	        	  TraficSource s=new TraficSource();
	    	        	  boolean ok=true;
	    	        	  s.idClient=rnd.nextInt(rw[0].nbrCar()-1) + 100;
	    	        	  s.idServer=rnd.nextInt(rw[0].nbrCar()-1) + 100;
	    	        	  
	    	        	  if(s.idClient==s.idServer){
    	        			  j--;
    	        			  continue;	    	        		  
	    	        	  }
	    	        	  
	    	        	  for( Iterator it=traficSources.iterator() ; it.hasNext() ; ){
	    	        		  TraficSource t=(TraficSource)it.next();
	    	        		  if((t.idServer==s.idServer) || (t.idClient==s.idClient)){
	    	        			  j--;
	    	        			  ok=false;
	    	        			  break;
	    	        		  }

	    	        		  
	    	        	  }
	    	        	  
	    	        	  if(ok)
	    	        		  traficSources.add(s);
	    	          }	    	  		
	    	  		
		    	  		/*int marge=(int)((options.speedTarget*options.timeSim)*1.5);
		    	  		RunWay runway=rw[options.doubleMind?1:0];

				    	 for (RunWayCar car=runway.beginEnumCar(0);car!=null;car=runway.enumCar(0, car)){
				    		 Location loc=runway.getCarLocation(car);
				    		 if(loc.getX()>(sizeRw-marge)){
				    			 idClient=car.id.intValue();
				    			 break;
				    		 }
				    	 }
				    		 
				    	 for (RunWayCar car=runway.beginEnumCar(0);car!=null;car=runway.enumCar(0, car)){
				    		 Location loc=runway.getCarLocation(car);
				    		 if(loc.getX()>(1000)){
				    			 server_id=car.id.intValue();
				    			 break;
				    		 }
				    	 }   */
	    	  		/*Random rnd=new Random();
	    	  		idClient= rnd.nextInt(rw[0].nbrCar()-1) + 100;
	    	  		server_id= rnd.nextInt(rw[0].nbrCar()-1) + 100;		   */ 	  	
	    	  	}
	    	  	
	    	  	//System.out.println(rw[0]);
	    	  	/*
	    	  	  if(i%5==0){
	    	  
		    	  	  place[0].reset();
				      while(addMobileNode(field, id,place[0],radioInfoShared,protMap,pl,pl,stats,udpEntity,rw[0],statsRw))
				    	  id++;
	    	  	  }
			      
			      rw[0].next(field);
			      
			      for(int j=0;j<rw[0].getNbrLan();j++){
			    	 for (RunWayCar car=rw[0].beginEnumCar(j);car!=null;car=rw[0].enumCar(j, car)){
			    		 if (car.id.intValue()==75)
			    			 System.out.println(car.loc);
			    	 }
			    		 
			      }
			      
			      RadioData rd=field.getRadioData(new Integer(75));
			      System.out.println(rd.getLocation());
			      
			      
			      System.out.println(jist.runtime.JistAPI.getTime()/Constants.MILLI_SECOND);
	      */
	      }
	      	      
	      
	      //addCBR(udpEntity,95);
	}
	
	

	
	  public static void main(String args[])
	  {
		try{  
	      final CommandLineOptions options = parseCommandLineOptions(args);
	      if(options.help) 
	      {
	        showUsage();
	        return;
	      }	
	      
	      if(options.verbose)
	      {
	    	 System.out.println(options); 
	      }
	      
	      
		  
		  hybridrunway sim= new hybridrunway();
		  sim.options= options;
	   //buildFieldTrio();
		  sim.buildFieldRunway();
		  
	    }
	    catch(CmdLineParser.OptionException e)
	    {
	      System.out.println(e.getMessage());
	    }
	  }	
}
