package driver;

import java.util.Random;

import jargs.gnu.CmdLineParser;
import jist.swans.misc.Location;
import jist.swans.runway.Placement;
import jist.swans.runway.RunWay;
import jist.swans.runway.graph.Graph;
import jist.swans.runway.graph.PathMonitor;

public class lifetime {
	
	static final float PRECISION=0.05f;
	static final int NB_ITER_INIT_RW = 1000;
	static final float TARGET_SPEED = 33.34f;
	
	  private static class CommandLineOptions
	  {
		  int nbPath;
		  int simTime;
		  int interCheckPaths;
		  double dAP;
		  int routeLenght;
		  int intensity;
	
		  public String toString(){
			  String s=new String();
			  
			  s+="Nombre de chemin générés : " + nbPath + "\n";
			  s+="Temps de simulation : " + simTime + "\n";
			  s+="Intervalle de verification de chemin : " + interCheckPaths + "\n";
			  s+="Densité d'AP : " + dAP + "\n";
			  s+="Taille de l'autoroute : " + routeLenght + "\n";
			  s+="Intensité du réseau de vehicule : " + intensity + "\n";
			  
			  return s;
		  }
	  }
	
	  private static CommandLineOptions parseCommandLineOptions(String[] args)
	    throws CmdLineParser.OptionException
	  {

	    CmdLineParser parser = new CmdLineParser();
	    CmdLineParser.Option opt_help = parser.addBooleanOption('h', "help");
	    CmdLineParser.Option opt_nbPath = parser.addIntegerOption('p', "nbPath");
	    CmdLineParser.Option opt_simTime = parser.addIntegerOption('t', "simTime");
	    CmdLineParser.Option opt_interCheckPaths = parser.addIntegerOption('I', "interCheckPaths");
	    CmdLineParser.Option opt_dAP = parser.addDoubleOption('a', "dAP");
	    CmdLineParser.Option opt_routeLenght = parser.addIntegerOption('l', "routeLenght");
	    CmdLineParser.Option opt_intensity = parser.addIntegerOption('i', "intensity");
	    
	    CommandLineOptions cmdOpts = new CommandLineOptions();
	    
	    parser.parse(args);
	    
	    if (parser.getOptionValue(opt_nbPath) != null){
	    	cmdOpts.nbPath = ((Integer)parser.getOptionValue(opt_nbPath)).intValue();
	    }else cmdOpts.nbPath = 1000;
	    if (parser.getOptionValue(opt_simTime) != null){
	    	cmdOpts.simTime = ((Integer)parser.getOptionValue(opt_simTime)).intValue();
	    }else cmdOpts.simTime = 60;
	    if (parser.getOptionValue(opt_interCheckPaths) != null){
	    	cmdOpts.interCheckPaths = ((Integer)parser.getOptionValue(opt_interCheckPaths)).intValue();
	    }else cmdOpts.interCheckPaths = 1500;
	    if (parser.getOptionValue(opt_dAP) != null){
	    	cmdOpts.dAP = ((Double)parser.getOptionValue(opt_dAP)).doubleValue();
	    }else cmdOpts.dAP = 0;
	    if (parser.getOptionValue(opt_routeLenght) != null){
	    	cmdOpts.routeLenght = ((Integer)parser.getOptionValue(opt_routeLenght)).intValue();
	    }else cmdOpts.routeLenght = 10000;
	    if (parser.getOptionValue(opt_intensity) != null){
	    	cmdOpts.intensity = ((Integer)parser.getOptionValue(opt_intensity)).intValue();
	    }else cmdOpts.intensity = 8;
	    
	    
	    return cmdOpts;
	  }

	  
	  
	public static void main(String[] args) throws CmdLineParser.OptionException {
		// TODO Auto-generated method stub
		  CommandLineOptions options;
		  
		  options = parseCommandLineOptions(args);
		  
		  System.err.println(options);
		  
		  
		  RunWay rw=new RunWay(3,3,options.routeLenght,TARGET_SPEED,options.intensity,PRECISION,true,new Location.Location2D(1,4));
		  Placement p = new Placement(rw);
		  rw.setPlacement(p);
		  rw.next();
		  rw.setCirculaire(true);
		  
		  System.err.println("Creation du graphe ");
		  
		  // nombre d'ap = [temp de simulation total (initialisation + simulation effective)] x [2 (marge)] 
		  //				x  [vitesse moyenne d'un vehicule] x [taille de la route] x  [densité d'AP] 
		  
		  Graph gutil=new Graph(rw,options.dAP, (int)((( (TARGET_SPEED*(options.simTime+(double)NB_ITER_INIT_RW*PRECISION)*2.0f)  + (double)options.routeLenght)/1000.0f)*options.dAP) );
		  edu.uci.ics.jung.graph.Graph g = gutil.getGraphe();
		  PathMonitor monitor = new PathMonitor(g,options.nbPath);
		  

		  
		  Random r=new Random();
		  
		  //System.out.println(g);
		  
		  System.err.println("Initialisation de la route");
		  for(int i=0;i<NB_ITER_INIT_RW;i++)
			  rw.next();
		  
		  System.err.println("Initialisation du graphe");
		  gutil.updateGraphe();
		  
		  int mid = gutil.l.size()/2;
		  
		  System.err.println("Initialisation des chemins ");
		  
		  for(int i=1;i<=options.nbPath;i++){
			  monitor.addPath(gutil.l.get(r.nextInt(gutil.l.size())) , gutil.l.get(r.nextInt(gutil.l.size())) );		  
		  }
		  System.err.println();

		  int nb_iter = (int)((float)options.simTime/PRECISION);
		  int intercheck = (int)((float)options.interCheckPaths/(PRECISION*1000f));
		  
		  for(int i=0;i<nb_iter;i++){
			  //edu.uci.ics.jung.graph.Graph g = gutil.getGraphe();
			  //System.out.println(g);

			  //System.out.println(gutil.getShortestPath(5, 50));
			  
			  
			  
			  
			  if(i%intercheck == 0){
				  System.err.print("Simulation "+((((double)i)/(double)nb_iter)*100.0f)+" %                                                \r");
				  gutil.updateGraphe();
				  //System.out.println(i*0.05f);
				  monitor.checkAllPath();	
			  }
			  
			  rw.next();

		  }
		  monitor.showStat();
		  
		  //System.err.println(monitor);
	/*        	
			final VisualizationViewer vv = new VisualizationViewer(new FRLayout(g),
			        new PluggableRenderer());
			
			final DefaultModalGraphMouse graphMouse = new DefaultModalGraphMouse();
			vv.setGraphMouse(graphMouse);
			
			//final ScalingControl scaler = new CrossoverScalingControl();	     
			
	        JPanel jp = new JPanel();
	        jp.setBackground(Color.WHITE);
	        jp.setLayout(new BorderLayout());
	        jp.add(vv, BorderLayout.CENTER);			

	        JFrame jf = new JFrame();
	        jf.getContentPane().add(jp);

	        jf.pack();
	        jf.setVisible(true);
	        
		*/  

	}

}
