//////////////////////////////////////////////////
// JIST (Java In Simulation Time) Project
// Timestamp: <dsrtest.java Tue 2004/04/06 11:57:46 barr pompom.cs.cornell.edu>
//

// Copyright (C) 2004 by Cornell University
// All rights reserved.
// Refer to LICENSE for terms and conditions of use.

package driver;

import jist.runtime.JistAPI;
import jist.swans.Constants;
import jist.swans.misc.Util;
import jist.swans.misc.Mapper;
import jist.swans.misc.Location;
import jist.swans.field.Fading;
import jist.swans.field.Field;
import jist.swans.field.Mobility;
import jist.swans.field.PathLoss;
import jist.swans.field.Spatial;
import jist.swans.radio.RadioNoise;
import jist.swans.radio.RadioNoiseIndep;
import jist.swans.radio.RadioInfo;
import jist.swans.bus.BusEthernetSwitch;
import jist.swans.mac.MacAddress;
import jist.swans.mac.Mac802_11;
import jist.swans.mac.MacDumbBus;
import jist.swans.net.NetAddress;
import jist.swans.net.NetIp;
import jist.swans.net.NetMultiInterface;
import jist.swans.net.PacketLoss;
import jist.swans.trans.TransUdp;
import jist.swans.app.AppJava;
import jist.swans.route.RouteMFR;
import jist.swans.loc.LocationGod;
import jist.swans.loc.LocationServiceInterface;
import jist.swans.field.Gps;

import java.net.InetAddress;
import java.net.DatagramSocket;
import java.net.DatagramPacket;

public class mfrtest
{

  public static final byte[] SERVER_IP = {0, 0, 0, 5};
  public static final int PORT = 3001;

  public static class Server
  {
    public static void main(String args[])
    {
      try
      {
        System.out.println("server starting at t="+JistAPI.getTime());
        DatagramSocket socket = new DatagramSocket(PORT);
        while(true){
	        
	        byte buf[] = new byte[256];
	        DatagramPacket packet = new DatagramPacket(buf, buf.length);
	        socket.receive(packet);
	        
	        System.out.println("received at t="+JistAPI.getTime()+
	            " ("+packet.getLength()+" bytes) "
	            +(new String(packet.getData(), packet.getOffset(), packet.getLength())));
        }
        //socket.close();
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }

  public static class Client
  {
    public static void main(String args[])
    {
      try
      {
        System.out.println("client starting at t="+JistAPI.getTime());
        
        for(int i=0;i<10;i++){
        
	        DatagramSocket socket = new DatagramSocket();
	        byte[] buf = "hi".getBytes();
	        DatagramPacket packet = new DatagramPacket(
	          buf,
	          buf.length,
	          InetAddress.getByAddress(SERVER_IP),
	          PORT);
	
	        System.out.println("sent at t="+JistAPI.getTime());
	        System.out.flush();
	        socket.send(packet);
	        socket.close();
	        
	        JistAPI.sleep(Constants.SECOND);
        
        }
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }
  static Field field = new Field(new Location.Location2D(200, 2500));
  static RadioNoise radio3;
  static RadioNoise radio4;
  
  
  private static NetIp CreateNode(int id,RadioInfo.RadioInfoShared radioInfoShared,Mapper protMap,Field field,PacketLoss pl,Location pos,LocationServiceInterface loc){
      RadioNoise radio = new RadioNoiseIndep(id, radioInfoShared);

      
      // mac
      Mac802_11 mac = new Mac802_11(new MacAddress(id), radio.getRadioInfo());
      // net
      NetIp net = new NetIp(new NetAddress(id), protMap, pl, pl);

      

      // route
      RouteMFR mfr = new RouteMFR(net.getAddress());

      
      Gps gps = new Gps(field,radio.getRadioInfo().getUnique().getID());

      
      gps.start();

      
      mfr.setGpsEntity(gps.getProxy());

      



      loc.addNode(net.getAddress(), radio.getRadioInfo().getUnique().getID());
      
      
      mfr.setLocService(loc);
  
      
      
      



      
      field.addRadio(radio.getRadioInfo(), radio.getProxy(), pos);
     
      // radio hookup
      radio.setFieldEntity(field.getProxy());
  

      radio.setMacEntity(mac.getProxy());
      
      // route hookup
      mfr.setNetEntity(net.getProxy());

      
      // net hookup
      byte intId = net.addInterface(mac.getProxy());

      
      net.setProtocolHandler(Constants.NET_PROTOCOL_MFR, mfr.getProxy());  
      
      net.setRouting(mfr.getProxy());

      // mac hookup
      mac.setRadioEntity(radio.getProxy());

      
      mac.setNetEntity(net.getProxy(), intId);

      

	  return net;
  }
  
  private static void MultiNodeTest()
  {
   try
    {
      // field
      //Location bounds = new Location.Location2D(200, 2500);
      
 
      
      RadioInfo.RadioInfoShared radioInfoShared = RadioInfo.createShared(
          Constants.FREQUENCY_DEFAULT, 
          Constants.BANDWIDTH_DEFAULT,
          Constants.TRANSMIT_DEFAULT, 
          Constants.GAIN_DEFAULT,
          Util.fromDB(Constants.SENSITIVITY_DEFAULT), 
          Util.fromDB(Constants.THRESHOLD_DEFAULT),
          Constants.TEMPERATURE_DEFAULT, 
          Constants.TEMPERATURE_FACTOR_DEFAULT, 
          Constants.AMBIENT_NOISE_DEFAULT);
      

      // protocol mapper
      Mapper protMap = new Mapper(Constants.NET_PROTOCOL_MAX);
      protMap.mapToNext(Constants.NET_PROTOCOL_MFR);
      protMap.mapToNext(Constants.NET_PROTOCOL_UDP);
      // packet loss
      PacketLoss pl = new PacketLoss.Zero();
      // nodes
      int i=1;
      // radio
      
      //create the location service's
      LocationGod loc = new LocationGod();
      loc.setFieldEntity(field);
      
      NetIp net1=CreateNode(1,radioInfoShared,protMap,field,pl,new Location.Location2D(0,0),loc.getProxy());
      NetIp net2=CreateNode(2,radioInfoShared,protMap,field,pl,new Location.Location2D(400,0),loc.getProxy());
      NetIp net3=CreateNode(3,radioInfoShared,protMap,field,pl,new Location.Location2D(600,100),loc.getProxy());
      NetIp net4=CreateNode(4,radioInfoShared,protMap,field,pl,new Location.Location2D(800,0),loc.getProxy());
      NetIp net5=CreateNode(5,radioInfoShared,protMap,field,pl,new Location.Location2D(1200,0),loc.getProxy());

      // trans
      TransUdp udp1 = new TransUdp();
      //TransUdp udp2 = new TransUdp();
      TransUdp udp2 = new TransUdp();
      
      // trans hookup
      udp1.setNetEntity(net2.getProxy());
     // udp2.setNetEntity(net2.getProxy());
      udp2.setNetEntity(net5.getProxy());  
      
      net2.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp1.getProxy());
      //net2.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp2.getProxy());
      net5.setProtocolHandler(Constants.NET_PROTOCOL_UDP, udp2.getProxy());      
      
      // applications
      AppJava server = new AppJava(Server.class);
      server.setUdpEntity(udp2.getProxy());
      AppJava client = new AppJava(Client.class);
      client.setUdpEntity(udp1.getProxy());

      // run apps
      server.getProxy().run(null); 
      JistAPI.sleep(1);
      client.getProxy().run(null);
      
  //    JistAPI.sleep(Constants.SECOND * 5 );
  
      
      JistAPI.runAt(new Runnable()
      {
        public void run()
        {        	  
            field.getProxy().moveRadioOff(new Integer(4), new Location.Location2D(0,200) );
            //field.getProxy().moveRadioOff(radio4.getRadioInfo().getUnique().getID(), new Location.Location2D(0,100) );
        }
	}, Constants.SECOND * 5);         	  
      
      
       
      
    }
    catch(Exception e) { e.printStackTrace(); }
  }


  public static void main(String args[])
  {
    //TwoNodeTest();
    MultiNodeTest();
  }
}
