package jist.swans.bus;

import jist.runtime.JistAPI.Proxiable;
import jist.swans.misc.Message;
import jist.swans.mac.MacAddress;

public interface BusInterface extends Proxiable {
	  //////////////////////////////////////////////////
	  // communication (mac)
	  //

	  /**
	   * Start transmitting message. Puts radio into transmission mode and contacts
	   * other radios that receive the signal. Called from mac entity.
	   *
	   * @param msg message object to transmit
	   * @param delay time to the wire
	   * @param duration time on the wire
	   */
	  void transmit(Message msg, long delay, long duration, MacAddress nextHop);

	  /**
	   * End message transmission. Putting the radio back into idle (or possibly
	   * receiving) mode. Called from mac entity.
	   */
	  void endTransmit();

	  //////////////////////////////////////////////////
	  // communication (field)
	  //

	  /**
	   * Start receiving message. Puts radio into receive or sensing mode depending
	   * on the message power and the state of the radio. A radio that is currently
	   * transmitting will ignore incoming messages. Called from field entity.
	   *
	   * @param msg incoming message
	   * @param power signal strength of incoming message (units: mW)
	   * @param duration time until end of transmission (units: simtime)
	   */
	  void receive(Message msg, Long duration);

	  /**
	   * End message reception. Puts the radio back into sensing or idle mode, and
	   * sends the received message to upper layers for processing, if no error has
	   * occurred during the reception. Called from field entity.
	   *
	   * @param power signal strength of incoming message (units: mW)
	   */
	  void endReceive();



}
