package jist.swans.route;

import jist.swans.mac.MacAddress;
import jist.swans.misc.Message;
import jist.swans.net.NetAddress;
import jist.swans.net.NetMessage;
import jist.swans.net.NetInterface;
import jist.swans.net.NetMultiInterface;
import jist.swans.net.NetIp;
import jist.swans.route.RouteDhg.DsrPacketStats;
import jist.swans.route.RouteDhg.DsrStats;
import jist.swans.route.RouteDsrBase.MaintenanceBufferEntry;
import jist.swans.route.RouteDsrMsg;
import jist.swans.trans.TransUdp;
import jist.minisim.hello_loop;
import jist.runtime.JistAPI;
import jist.swans.Constants;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.apache.log4j.Logger;




public class RouteDhg extends RouteDsrBase implements RouteInterface.Dhg,  RouteInterface.Dsr{

	  /** The maximum amount of jitter before sending a packet. */ 
	  public static final long REGISTER_JITTER       = 10 * Constants.MILLI_SECOND;	
	  

	/** true if is a AP*/
	private boolean AP;
	
	private boolean regReactif=false;
	private boolean bbProactif=false;
	private boolean idle=false;

	/** proxy interface*/
	private RouteInterface.Dhg self;
	
	/** Wireless interfaces*/
	protected NetInterface  netEntityWireless	;
	
	/** List of Wired interface*/
	protected NetInterface netEntitiesWired[];
	
	/** Set of couple Interface/Ip address */
	private Hashtable routingTable;
	
	/** TTL for one cellule */
	private byte diameter;
	
	/**AP selected */
	private APTable.APEntry apSelected;
	
	protected NetMultiInterface  netMI;
	
	
	/** time between two advertissement */
	private long advertissementInter=(int)((double)Constants.SECOND);

	/** time between two advertissement */
	private long helloInter=(int)((double)100 * Constants.MILLI_SECOND);	
	
	/** table of APs*/
	APTable apTable;

	/** DHG logger. */
	private static Logger log = Logger.getLogger(RouteDhg.class.getName());

	protected DsrStats stats;
	  
	public static class DsrPacketStats extends RouteDsrBase.DsrPacketStats
	{
	    /** RREG Packet */
	    public long rregPackets;
	    public long rregSizePackets;
	    
	    
	    /** Ap Advertissement Packet */
	    public long apAdvPackets;
	    public long apAdvSizePackets;
    
	    
	    
	    /** Ap Search Packet */
	    public long apSearchPackets;
	    public long apSearchSizePackets;	    
	    
	   
		
	    /** {@inheritDoc} */
	    public void clear()
	    {
	    	
	    	apSearchPackets=0;
	    	apSearchSizePackets=0;	    	
	    	rregPackets=0;
	    	rregSizePackets=0;
	    	apAdvPackets=0;
	    	apAdvSizePackets=0;
	    	super.clear();
	    }	    
	    
	    /** {@inheritDoc} */
	    public String toString(){
	    	String s=super.toString();
	    	
	    	s+= "Nombre de paquet  RREG: " + rregPackets + " (" + rregSizePackets + ")\n";
	    	s+= "Nombre de paquet  AP Advsertissement: " + apAdvPackets + " (" + apAdvSizePackets + ")\n";
	    	s+= "Nombre de paquet  AP Search: " + apSearchPackets + " (" + apSearchSizePackets + ")\n";

	    	
	    	return s;
	    }
	
	}
	
	public static class DsrStats extends RouteDsrBase.DsrStats{
		
		public DsrStats(){
			super.send=send;
			super.recv=recv;
		}
		
	    /** sent packets. */
	    public DsrPacketStats send = new DsrPacketStats();
	    /** received packets. */
	    public DsrPacketStats recv = new DsrPacketStats();	

	    
	 /*   public String toString(){
	    	String s="";
	    
	    	
	    	s+="Statistique des packet envoyé:\n";
	    	s+=send;
	    	
	    	s+="Statistique des packet recu:\n";
	    	s+=recv;  
	    	
	    	
	        s+="Nombre de paquet de la couche transport : "+netMsgs+"\n";
	        s+="Nombre d'origine RREQ : "+rreqOrig+"\n";
	        s+="Nombre d'origine RREP : "+rrepOrig+"\n";
	        s+="Nombre de RREQ succès : "+rreqSucc+"\n";
	        
	        s+="Longueur de la route moyenne : "+avgLength() +"\n";
	        
	        return s;
	    }*/	    
	    
	}
	
	/**
	 * AP's Table (set of APEntry)
	 * 
	 * @author florent
	 *
	 */
	private class APTable{
		/**
		 * Entry for one AP in a MN
		 * 
		 * @author florent
		 *
		 */
		private class APEntry{
			/** AP's address*/
			NetAddress addr;
			/** number of hop to the AP*/
			byte hop;
			/** add's time */
			long time;
			
			/** RouteRequest option of AP Advertissement 
			 * (used for get the route to the AP) */
			NetAddress[] addrs;
			
			public boolean linkIsInRoute(NetAddress addr1,NetAddress addr2){
				
				
				
				if(addr1.equals(localAddr) && addrs.length>0 && addr2.equals(addrs[addrs.length-1]))
					return true;

				if(addr1.equals(localAddr) && addrs.length==0  && addr2.equals(addr))
					return true;

				if(addrs.length>0 && addr1.equals(addrs[0]) && addr2.equals(addr))
					return true;		
				
		         for (int i = 1; i < addrs.length; i++)
		         {
		            if (addr1.equals(addrs[i]) && addr2.equals(addrs[i-1]))
		            	return true;
		         }

				return false;
			}
			
			public boolean equals(Object o)
			{
				if(this==o) return true;
				if(o==null) return false;
				if(!(o instanceof APEntry)) return false;
				
				APEntry ap=(APEntry)o;
				
				if( ! ap.addr.equals(this.addr)) return false;
				
				if(ap.addrs.length != this.addrs.length) return false;
				
				for(int i=0;i<ap.addrs.length;i++)
					if( ! ap.addrs[i].equals(this.addrs[i])) 
						return false;
				
				return true;
			}
			
			public String toString(){
				
				String s="";
				
				s=" AP "+addr + " Regat "+time+" RouteToPath "+hop+" {";

				for(int i=0;i<addrs.length;i++)
					s+=addrs[i]+" ";
				
				s+="}";
				
				return s;
				
			}

			
			APEntry(NetAddress addr,byte hop,long time,NetAddress[] addrs){
				this.addr=addr;
				this.hop=hop;
				this.time=time;
				this.addrs=addrs;
			}
			
			
			
			
		}
		
		HashSet APs;
		
		public APTable(){
			
			APs = new HashSet();
		}
		
		void add(NetAddress addr,byte hop,NetAddress[] addrs){
			Iterator it=APs.iterator();


		  	if (log.isInfoEnabled())
		 	    log.info( JistAPI.getTime() + " " + localAddr + " AP Table : add "+addr+" AP at "+hop+" hops  path : "+routeToString(addrs));				
			
			while(it.hasNext()){
				APEntry ap=(APEntry)it.next();
				
				if(ap.equals(this))
					it.remove();
		
			}
			
			
			
			APs.add(new APEntry(addr,hop,JistAPI.getTime(),addrs));
		}			
		
		void add(NetAddress addr,byte hop,RouteDsrMsg.OptionRouteReply rRep){

			NetAddress[] addrs =  new NetAddress[rRep.getNumAddresses()-2];
			
			for (int i=0;i< addrs.length;i++)
				addrs[i]=rRep.getAddress(addrs.length - i);
			
			add(addr,hop,addrs);
		}		
		
		void add(NetAddress addr,byte hop,RouteDsrMsg.OptionRouteRequest rReq){

			NetAddress[] addrs =  new NetAddress[rReq.getNumAddresses()];
			
			for (int i=0;i< addrs.length;i++)
				addrs[i]=rReq.getAddress(i);
			
			add(addr,hop,addrs);
		}
		
		NetAddress[] getRouteAp(NetAddress ap){
			Iterator it=APs.iterator();		
			
			while(it.hasNext()){
				APEntry i=(APEntry)it.next();
				
				if(i.addr.equals(ap))
					return i.addrs;
		
			}			
			return null;			
		}
		
		void removeApInvalid(NetAddress addr1,NetAddress addr2){
			Iterator it=APs.iterator();


			
			
			while(it.hasNext()){
				APEntry i=(APEntry)it.next();
				
				if(i.linkIsInRoute(addr1, addr2)){
	    			if(log.isInfoEnabled())
	    				log.info(JistAPI.getTime()+" Remove_AP_Entry "+i);
					it.remove();
				}
		
			}			
			
		}
		
		boolean isInListAP(APEntry ap){
			Iterator it=APs.iterator();


			
			
			while(it.hasNext()){
				APEntry i=(APEntry)it.next();
				
				if(i.equals(ap))
					return true;
		
			}			
			return false;
		}
		
		/**
		 * Compute the nearer AP
		 *
		 */
		 APEntry computeNearerAP(){
			Iterator it=APs.iterator();
			APEntry apSelected=null;
			
			while(it.hasNext()){
				APEntry ap=(APEntry)it.next();
				
				if((JistAPI.getTime()-ap.time)>((1.5*(float)advertissementInter))){
				  	if (log.isDebugEnabled())
				 	    log.debug("remove AP "+ap.addr);
					it.remove();	
				}else	
					//if the minimum hop and AP advertissement fresh 
					if((apSelected==null || apSelected.hop>ap.hop)){
						apSelected=ap;
					}
			}
			
			return apSelected;
		}
		 
		 public String toString(){
			 
				Iterator it=APs.iterator();
				
				String s="";
				
				while(it.hasNext()){
					APEntry ap=(APEntry)it.next();

					s+=JistAPI.getTime()+" "+" APEntry "+ap+"\n";
				}			 
			 
			 return s;
		 }
		
	}
	

	
	
	public RouteDhg(NetAddress localAddr){
		this(localAddr,false);
		
	}
	
	public RouteDhg(NetAddress localAddr,boolean AP){
		super(localAddr);	
		this.AP=AP;
		self = (RouteInterface.Dhg)JistAPI.proxy(this, RouteInterface.Dhg.class);
		diameter = 8;
		apTable=new APTable();
		//if(AP)
			self.RequestPosTimeOut();

			
		if(!AP)
			self.registerTimeOut();
			
		netEntitiesWired = new NetInterface[0];
		routingTable = new Hashtable();
		
		netMI = new NetMultiInterface();
		
		//call the methode of DSR
		setNetEntity(netMI);	
		
	}
	
	
	
	
	  /**
	   * Sends the given message and looks for acknowledgement.  If no
	   * acknowledgement is forthcoming the message is retransmitted a limited
	   * number of times.
	   *
	   * @param netEntity net entity
	   * @param msg the message to send.  The payload of this IP packet should be
	   *   a <code>RouteDsrMsg</code>, and it should not already contain an
	   *   Acknowledgement Request.
	   */
	  protected void Transmit(NetInterface netEntity,NetMessage.Ip msg)
	  {
	    RouteDsrMsg dsrMsg = (RouteDsrMsg)msg.getPayload();
	    RouteDsrMsg.OptionSourceRoute sourceRoute = GetSourceRoute(dsrMsg);

	    if (sourceRoute == null || sourceRoute.getNumSegmentsLeft() == 0)
	    {
	      // Messages on their last hop must use network-level acknowledgements.
	      // Add an Acknowledgement Request to the packet.
	      RouteDsrMsg newMsg = (RouteDsrMsg)dsrMsg.clone();

	      Short ackId = new Short(nextAckId++);
	      newMsg.addOption(RouteDsrMsg.OptionAckRequest.create(ackId.shortValue()));
	      activeAcks.add(ackId);
	      
	  

	      NetMessage.Ip ipMsg = new NetMessage.Ip(newMsg, msg.getSrc(),
	        msg.getDst(), msg.getProtocol(), msg.getPriority(), msg.getTTL(),
	        msg.getId(), msg.getFragOffset());

	      self.TransmitWithNetworkAck(netEntity,ipMsg, ackId, MAINT_PERIOD, 0);
	      return;
	    }

	    // Otherwise add an entry to the Maintenance Buffer and try passive
	    // acknolwedgements
	    MaintenanceBufferEntry entry = new MaintenanceBufferEntry(msg.getSrc(),
	      msg.getDst(), msg.getProtocol(), msg.getId(), msg.getFragOffset());

	    maintenanceBuffer.put(entry, new Integer(sourceRoute.getNumSegmentsLeft()));
	    self.TransmitWithPassiveAck(netEntity,msg, 0);
	  }

	  
	  /**
	   * Sends the given message and waits for a passive acknowledgement.  If no
	   * acknowledgement is heard, the message will be retransmitted up to
	   * <code>TRY_PASSIVE_ACKS</code> times.
	   *
	   * @param msg            the message to be sent
	   * @param numRetransmits the number of times this message has already been
	   *   retransmitted.  Callers should usually pass in zero for this.
	   */
	  public void TransmitWithPassiveAck(NetInterface netEntity,NetMessage.Ip msg, int numRetransmits)
	  {
	    RouteDsrMsg dsrMsg = (RouteDsrMsg)msg.getPayload();
	    RouteDsrMsg.OptionSourceRoute sourceRoute = GetSourceRoute(dsrMsg);
	    MaintenanceBufferEntry entry = null;
	    
	    if (log.isDebugEnabled())
	    {
	        log.debug(localAddr + " TransmitWithPassiveAck " + numRetransmits);
	    }  	      

	    if (sourceRoute != null && sourceRoute.getNumSegmentsLeft() > 0)
	    {
	      entry = new MaintenanceBufferEntry(msg.getSrc(), msg.getDst(),
	        msg.getProtocol(), msg.getId(), msg.getFragOffset());

	      // If the maintenance buffer does not contain this entry, then it
	      // has already been sent and acknowledged
	      if (!maintenanceBuffer.containsKey(entry))
	      {
	        return;
	      }

	      // If we have exceeded the maximum number of tries on passive
	      // acknowledgement, remove this entry from the maintenance buffer
	      if (numRetransmits >= TRY_PASSIVE_ACKS)
	      {
	        maintenanceBuffer.remove(entry);
	      }
	    }

	    if (sourceRoute == null || sourceRoute.getNumSegmentsLeft() == 0 ||
	        numRetransmits >= TRY_PASSIVE_ACKS)
	    {
	      // Messages on their final hop and messages that have already tried and
	      // failed to receive a passive acknowledgement must use network-level
	      // acknowledgments
	      RouteDsrMsg newMsg = (RouteDsrMsg)dsrMsg.clone();

	      Short ackId = new Short(nextAckId++);
	      newMsg.addOption(RouteDsrMsg.OptionAckRequest.create(ackId.shortValue()));
	      activeAcks.add(ackId);
	      
	   

	      NetMessage.Ip ipMsg = new NetMessage.Ip(newMsg, msg.getSrc(),
	        msg.getDst(), msg.getProtocol(), msg.getPriority(), msg.getTTL(),
	        msg.getId(), msg.getFragOffset());

	      TransmitWithNetworkAck(ipMsg, ackId, MAINT_PERIOD, 0);
	      return;
	    }

	    /*Pas cool : quand on restransmet c'est forcement un paquet data*/
	    if (numRetransmits > 0)
	    {
	    	  stats.send.dsrPackets++; 	  
	      	  stats.send.dsrSizePackets+=msg.getSize();  
	      	  /*stats.send.dataPackets++;
	      	  stats.send.dataSizePackets +=msg.getSize();*/     	
	    }    
	    
	    netEntity.send(msg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
	    JistAPI.sleep(PASSIVE_ACK_TIMEOUT + (long)(Math.random()*BROADCAST_JITTER));

	    self.TransmitWithPassiveAck(msg, numRetransmits + 1);
	  }  
	  
	  /**
	   * Sends the given message and waits for a passive acknowledgement.  If no
	   * acknowledgement is heard, the message will be retransmitted up to
	   * <code>TRY_PASSIVE_ACKS</code> times.
	   *
	   * @param msg            the message to be sent
	   * @param numRetransmits the number of times this message has already been
	   *   retransmitted.  Callers should usually pass in zero for this.
	   */
	  public void TransmitWithPassiveAck(NetMessage.Ip msg, int numRetransmits)
	  {
		  TransmitWithPassiveAck(netEntity,msg,numRetransmits);
	   }

	  
	  /**
	   * Sends the given message.  The message should be a DSR packet containing an
	   * acknowledgement request with the given id.  If no acknowledgement is
	   * received within the given timeout, the packet will be retransmitted up to
	   * <code>MAX_MAINT_REXMT</code> times.
	   *
	   * @param msg            the message to be sent
	   * @param ackId          the ID number of the Acknowledgement Request
	   * @param timeout        the number of clock ticks to wait before
	   *   retransmitting the message
	   * @param numRetransmits the number of times this packet has already been
	   *   transmitted.  Callers should normally pass in zero for this.
	   */
	  public void TransmitWithNetworkAck(NetMessage.Ip msg, Short ackId,
	                                     long timeout, int numRetransmits){
		  
		  TransmitWithNetworkAck(netEntity,msg,ackId,timeout,numRetransmits);
		  
	  }
	  
	  /**
	   * Sends the given message.  The message should be a DSR packet containing an
	   * acknowledgement request with the given id.  If no acknowledgement is
	   * received within the given timeout, the packet will be retransmitted up to
	   * <code>MAX_MAINT_REXMT</code> times.
	   *
	   * @param netEntity	   the net entity 
	   * @param msg            the message to be sent
	   * @param ackId          the ID number of the Acknowledgement Request
	   * @param timeout        the number of clock ticks to wait before
	   *   retransmitting the message
	   * @param numRetransmits the number of times this packet has already been
	   *   transmitted.  Callers should normally pass in zero for this.
	   */
	  public void TransmitWithNetworkAck(NetInterface netEntity,NetMessage.Ip msg, Short ackId,
	                                     long timeout, int numRetransmits)
	  {
	    if (log.isDebugEnabled())
	    {
	        log.debug(localAddr + " TransmitWithNetworkAck " + numRetransmits);
	    }  	  
		  
	    if (!activeAcks.contains(ackId)) return;

	    if (numRetransmits > MAX_MAINT_REXMT)
	    {
			
		    if (/*log.isDebugEnabled()*/true)
		    {
		    	RouteDsrMsg dsrMsg = (RouteDsrMsg)msg.getPayload();
		    	
    			if(log.isInfoEnabled())
    				log.info(JistAPI.getTime()+ " lien_rompu "+localAddr+" "+msg.getSrc()+" "+ msg.getDst()+" "+dsrMsg.hasOption(RouteDsrMsg.OPT_AP_REGISTRY) + " "+ dsrMsg.hasOption(RouteDsrMsg.OPT_SOURCE_ROUTE));

		    	
		    	Iterator i=dsrMsg.getOptions().iterator();

		    	    byte[] optBuf = (byte[])i.next();
		    	    RouteDsrMsg.Option opt = RouteDsrMsg.Option.create(optBuf, 0);
		    		
		    		if(opt.getType()==96){
		    			if(dsrMsg.getContent()!=null){
		    			byte[] contBuf = new byte[dsrMsg.getContent().getSize()];
		    			TransUdp.UdpMessage udpmsg = (TransUdp.UdpMessage) dsrMsg.getContent();
		    			udpmsg.getPayload().getBytes(contBuf,0);
		    			String strcont = new String(contBuf);
		    		
		    			if(log.isInfoEnabled())
		    				log.info(JistAPI.getTime()+ " " +localAddr + "paquet de donnee perdu "+ opt.getType() + " " + jist.runtime.JistAPI.getTime() + " "+ strcont.trim());
		    			}
		    		}
		    }       	
	    	
	      // Max retransmissions exceeded -- must send Route Error to message source
	      activeAcks.remove(ackId);

	      if (msg.getProtocol() == Constants.NET_PROTOCOL_DSR)
	      {
	        RouteDsrMsg dsrMsg = (RouteDsrMsg)msg.getPayload();
	        
	 
	        NetAddress unreachableAddress = null;

	        // Find out what the unreachable address was
	        if (!dsrMsg.hasOption(RouteDsrMsg.OPT_SOURCE_ROUTE))
	        {
	          unreachableAddress = msg.getDst();
	        }
	        else
	        {
	          Iterator iter = dsrMsg.getOptions().iterator();
	          while (iter.hasNext())
	          {
	            byte[] optBuf = (byte[])iter.next();
	            if (RouteDsrMsg.Option.getType(optBuf) == RouteDsrMsg.OPT_SOURCE_ROUTE)
	            {
	              RouteDsrMsg.OptionSourceRoute sourceRoute =
	                (RouteDsrMsg.OptionSourceRoute)RouteDsrMsg.Option.create(optBuf, 0);

	              unreachableAddress = NextRecipient(sourceRoute, msg.getDst());
	              break;
	            }
	          }
	        }

			if(log.isInfoEnabled())
				log.info(JistAPI.getTime()+ " " +localAddr + " RemoveCachedLink " + unreachableAddress);
	                
	        
	        // Remove the broken link from the cache, then try to retransmit the
	        // message if we know any other routes to the destination
	        RemoveCachedLink(localAddr, unreachableAddress);        

	        //remove th AP entry where use the link break
	        apTable.removeApInvalid(localAddr, unreachableAddress);
	        //register is necessary
	        if(!idle && !AP)
	        	registreToBestAP();
	        

	        if (msg.getSrc().equals(localAddr))
	        {
	          if (GetCachedRoute(msg.getDst()) != null) CheckBuffer(msg.getDst());
	        }
	        else
	        {
	        	
	        	
	          // Otherwise, must send Route Error message to the originator
	          // of this message
	          SendRouteError(dsrMsg, msg.getSrc(), msg.getDst());
	          
	          if(dsrMsg.getContent()!=null && SALVAGE){
	          
	        	  
	        	  NetAddress[] route = GetCachedRoute(msg.getDst());
		          
		          if (route != null) {

		    		
		    		Message payload=dsrMsg.getContent();
		    		
		    		//make the ip message
		    		NetMessage.Ip ipMsg = new NetMessage.Ip(payload, msg.getSrc(),
		    				msg.getDst(), dsrMsg.getNextHeaderType(), msg.getPriority(), msg.getTTL(),
		    				msg.getId(), msg.getFragOffset());

		    	    
		    		SendWithRoute(ipMsg, route);	    		

		          }else{
			          //packet's salvage  
			          super.self.InsertBuffer(msg);
			          activeRequests.add(msg.getDst());
			          DiscoverRoute(msg.getDst(), nextRequestId++);  
		          }
	          
	          }
	          
	        }
	      }
	      return;
	    }

	    if (log.isDebugEnabled())
	    {
	      if (numRetransmits > 0)
	      {
	        log.debug(localAddr + " type filter textretransmitting " + numRetransmits +  " from " + msg.getSrc() + " to " +
	          msg.getDst() + "!");
	      }
	    }
	    
	    /*Pas cool : quand on restransmet c'est forcement un paquet data*/
	    if (numRetransmits > 0)
	    {
	    	  stats.send.dsrPackets++; 	  
	      	  stats.send.dsrSizePackets+=msg.getSize();  
	      	  /*stats.send.dataPackets++;
	      	  stats.send.dataSizePackets +=msg.getSize();     */	
	    }

	    netEntity.send(msg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
	    JistAPI.sleep(timeout + (long)(Math.random() * BROADCAST_JITTER));
	    self.TransmitWithNetworkAck(netEntity,msg, ackId, 2*timeout, numRetransmits + 1);
	  }	
	
	
	
	
	/**
	 * Envoi un APAdv en reponse à un APSearch
	 * 
	 */
	private void ackReg(NetAddress src,RouteDsrMsg.OptionRouteRequest opt){
	    NetAddress[] routeToHere = new NetAddress[opt.getNumAddresses() + 2];
	    routeToHere[0] = src;
	    for (int i = 1; i < routeToHere.length-1; i++)
	    {
	      routeToHere[i] = opt.getAddress(i-1);
	    }
	    
	    routeToHere[routeToHere.length - 1] = localAddr;
	    
	    NetAddress[] routeFromHere = new NetAddress[routeToHere.length - 2];
	    for (int i = 0; i < routeFromHere.length; i++)
	    {
	      routeFromHere[i] = routeToHere[routeToHere.length - i - 2];
	    }
	    
	    // Add a Route Reply option indicating how to get here from the
	    // source and a Source Route option indicating how to get to the
	    // source from here.
	    RouteDsrMsg reply = new RouteDsrMsg(null);
	    reply.addOption(RouteDsrMsg.OptionRouteReply.create(routeToHere));
	    
	    
	    
	    if (routeFromHere.length > 0)
	    {
	      reply.addOption(RouteDsrMsg.OptionSourceRoute.create(0,
	        routeFromHere.length, routeFromHere));
	    }
		//add the Access Point Option
	    reply.addOption(RouteDsrMsg.OptionAccessPointAdvertissement.create());

	    
	    if(stats!=null){
	   	  /*stats.send.rrepPackets++;
	   	  stats.send.rrepSizePackets+=reply.getSize();*/
	   	  stats.send.dsrPackets++; 	 
	   	  stats.send.dsrSizePackets+=reply.getSize();
	   	  stats.rrepOrig++;
		  stats.send.apAdvPackets++;
		  stats.send.apAdvSizePackets+=reply.getSize();
	     } 
 
  	   if (log.isInfoEnabled())
 	        log.info(JistAPI.getTime()+" Ack Search "+src+" ("+localAddr+")");   

	    NetMessage.Ip replyMsg = new NetMessage.Ip(reply, localAddr,
	      src, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
	      Constants.TTL_DEFAULT);
	    

	    JistAPI.sleep((long)(Math.random() * BROADCAST_JITTER));
	    
	    //SEULEMENT sur le wireless
	    //Transmit(replyMsg);
		
	    netEntityWireless.send(replyMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);

	}
	
	private void searchAP(){

		
	  	   if (log.isInfoEnabled())
	 	        log.info(JistAPI.getTime()+" search a AP "+localAddr);

			//Create the RREQ
		      RouteDsrMsg routeRequest = new RouteDsrMsg(null);
		      routeRequest.addOption(RouteDsrMsg.OptionRouteRequest.create(nextRequestId++,
		    		  NetAddress.ANY, new NetAddress[0]));
		      
			//add the Access Point Option
		      routeRequest.addOption(RouteDsrMsg.OptionAccessPointSearch.create());

		      if(stats!=null){	    	        		
		    	  stats.rreqOrig++;
		    	  /*stats.send.rreqPackets++;
		    	  stats.send.rreqSizePackets+=routeRequest.getSize();*/
		    	  stats.send.dsrPackets++;
		    	  stats.send.dsrSizePackets+=routeRequest.getSize();    
		    	  stats.send.apSearchPackets++;
		    	  stats.send.apSearchSizePackets+=routeRequest.getSize();	  
		      }

		      NetMessage.Ip routeRequestMsg = new NetMessage.Ip(routeRequest, localAddr,
		        NetAddress.ANY, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
		        diameter);
		      

		      //send the message
		      netEntity.send(routeRequestMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
			
		
		
	}
	
	private void unregistreToAP(NetAddress ap){
		NetAddress[] route =  apTable.getRouteAp(ap);
		
		if(log.isInfoEnabled())
			log.info("unregistre AP " + ap);
		
		if(route!=null)
			SendRouteError(route, ap, localAddr, ap, 0, RouteDsrMsg.ERROR_OPTION_AP_UNREGISTRE);
		
		
	}
	
	private void registreToAP(NetAddress ap,NetAddress src,NetAddress[] addrs,boolean forwardBB){
	    NetAddress[] routeToHere = new NetAddress[addrs.length + 2];
	    routeToHere[0] = ap;
	    for (int i = 1; i < routeToHere.length-1; i++)
	    {
	      routeToHere[i] = addrs[i-1];
	    }
	    
	    routeToHere[routeToHere.length - 1] = localAddr;
	    
	    NetAddress[] routeFromHere = new NetAddress[routeToHere.length - 2];
	    for (int i = 0; i < routeFromHere.length; i++)
	    {
	      routeFromHere[i] = routeToHere[routeToHere.length - i - 2];
	    }
	    
	    // Add a Route Reply option indicating how to get here from the
	    // source and a Source Route option indicating how to get to the
	    // source from here.
	    RouteDsrMsg reply = new RouteDsrMsg(null);
	    reply.addOption(RouteDsrMsg.OptionRouteReply.create(routeToHere));
	    
	    
	    
	    if (routeFromHere.length > 0)
	    {
	      reply.addOption(RouteDsrMsg.OptionSourceRoute.create(0,
	        routeFromHere.length, routeFromHere));
	    }
	    
	    reply.addOption(RouteDsrMsg.OptionAccessPointRegistry.create(false,forwardBB));
	    
	    if(stats!=null){
	   	  /*stats.send.rrepPackets++;
	   	  stats.send.rrepSizePackets+=reply.getSize();*/
	   	  stats.send.dsrPackets++; 	 
	   	  stats.send.dsrSizePackets+=reply.getSize();
	   	  stats.rrepOrig++;
		  stats.send.rregPackets++;
		  stats.send.rregSizePackets+=reply.getSize();
	     } 
 
  	   if (log.isDebugEnabled())
 	        log.debug("Registre to the AP "+ap+" ("+localAddr+")");   
  	   
  	   
	    NetMessage.Ip replyMsg = new NetMessage.Ip(reply, localAddr,
	      ap, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
	      Constants.TTL_DEFAULT);
	    

	    JistAPI.sleep((long)(Math.random() * BROADCAST_JITTER));
	    
	    Transmit(netEntity,replyMsg);
	    //netEntity.send(replyMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);

	    //Transmit(replyMsg);
		
	}
	
	
	  /**
	   * Forwards a DSR packet without ack
	   *
	   * @param netEntity  the net entity
	   * @param msg        the <code>RouteDsrMsg</code> to be forwarded
	   * @param opt        the Source Route option
	   * @param optBuf     the bytes of the Source Route option
	   * @param src        the address of the originator of this packet
	   * @param dest       the address of the ultimate destination of this packet
	   * @param protocol   the IP protocol of this packet (usually DSR)
	   * @param priority   the IP priority of this packet
	   * @param ttl        the IP time to live of this packet
	   * @param id         the IP identification of this packet
	   * @param fragOffset the IP fragmentation offset of this packet
	   */
	  protected void ForwardPacketWithoutAck(NetInterface netEntity,RouteDsrMsg msg, RouteDsrMsg.OptionSourceRoute opt,
	                             byte[] optBuf, NetAddress src, NetAddress dest,
	                             short protocol, byte priority, byte ttl,
	                             short id, short fragOffset)
	  {
		   // If the packet is for me it doesn't need to be forwarded
		    if (localAddr.equals(dest)) return;
		    
		    
		    log.debug("ForwardPacket segmentleft : "+opt.getNumSegmentsLeft());
		    
		    // Check if I am the intended next recipient of this packet
		    if (localAddr.equals(NextRecipient(opt, dest)))
		    {
		    
    	
		    	
			    NetMessage.Ip ipMsg = GetForwardPacket(msg,opt,optBuf,src,dest,protocol,priority,ttl,id,fragOffset);
			    
			    netEntity.send(ipMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
			    
			    //Transmit(netEntity,ipMsg);
		    
		    }
	  
	  }	
	  
	  
	private void checkAPEntryValid(RouteDsrMsg.OptionRouteError opt){
	    switch (opt.getErrorType())
	    {
		
	    	case RouteDsrMsg.ERROR_NODE_UNREACHABLE:	
	            byte[] unreachableAddrBytes = new byte[4];
	            opt.getTypeSpecificInfoBytes(unreachableAddrBytes, 0);
	            NetAddress unreachableAddr = new NetAddress(unreachableAddrBytes);
	    		
	            //remove the AP concerned
	    		apTable.removeApInvalid(opt.getSourceAddress(),unreachableAddr);
	    		

	    		//in receive route error, register if necessary
	    		if(!idle && !AP)
	    			registreToBestAP();
	    		break;

	    	default:
	    		break;
	    }		
	}
	
	/**
	 * 
	 * Supprimer dans le cache les entrée concernant le noeud source du
	 * message de déenregistrement
	 * @param opt
	 */
	private void HandleRouteErrorUnregistre(RouteDsrMsg.OptionRouteError opt){
		
	    switch (opt.getErrorType())
	    {

	      case RouteDsrMsg.ERROR_OPTION_AP_UNREGISTRE:

	 
	        
	    	if(log.isDebugEnabled()){
	    		log.debug(localAddr+"Unregistre Remove Cached Link : "+opt.getSourceAddress());
    			log.debug("Route Cache : "+cacheToString());
	    	} 	
	    	  
	        routeCache.remove(opt.getSourceAddress());

	    	if(log.isDebugEnabled()){
	    		log.debug(localAddr+"Fin Unregistre Remove Cached Link : "+opt.getSourceAddress());	    		
	    		log.debug("Route Cache : "+cacheToString());	
	    	} 	        
	        
	        break;

	      default:
	        break;
	    }
		
		
	}
	
	private void HandleSearchAP(NetAddress src,RouteDsrMsg.OptionRouteRequest opt){
		ackReg(src,opt);
		
	}
	
	private RouteDsrMsg ForwardAPRegistry(RouteDsrMsg msg,NetAddress src,NetAddress dst,RouteDsrMsg.OptionRouteReply optRRep,RouteDsrMsg.OptionAccessPointRegistry optApReg,RouteDsrMsg.OptionSourceRoute sourceRoute){
		
		NetAddress[] routeToHere;	
		
		int nbr;
		
		if(optRRep.getAddress(0).equals(localAddr)){
			nbr=optRRep.getNumAddresses();
			
			routeToHere=new NetAddress[nbr];
			
			for(int i=0;i<nbr;i++){
				routeToHere[i]=optRRep.getAddress(i);
			}
		}else{
			nbr=optRRep.getNumAddresses()+1;
			
			routeToHere=new NetAddress[nbr];
			
			routeToHere[0]=localAddr;
			
			for(int i=1;i<nbr;i++){
				routeToHere[i]=optRRep.getAddress(i-1);
			}			
		}

		
				


		
	    NetAddress[] routeFromHere = new NetAddress[routeToHere.length - 2];
	    for (int i = 0; i < routeFromHere.length; i++)
	    {
	      routeFromHere[i] = routeToHere[routeToHere.length - i - 2];
	    }		
		
	    // Add a Route Reply option indicating how to get here from the
	    // source and a Source Route option indicating how to get to the
	    // source from here.
	    RouteDsrMsg reply = new RouteDsrMsg(null);
	    reply.addOption(RouteDsrMsg.OptionRouteReply.create(routeToHere));
	    
	    
	    
	    /*if (routeFromHere.length > 0)
	    {
	      reply.addOption(RouteDsrMsg.OptionSourceRoute.create(0,
	        routeFromHere.length, routeFromHere));
	    }*/
	    
	    reply.addOption(RouteDsrMsg.OptionAccessPointRegistry.create(true,true));
	    
	    if(stats!=null){
	   	  /*stats.send.rrepPackets++;
	   	  stats.send.rrepSizePackets+=reply.getSize();*/
	   	  stats.send.dsrPackets++; 	 
	   	  stats.send.dsrSizePackets+=reply.getSize();
	   	  stats.rrepOrig++;
		  stats.send.rregPackets++;
		  stats.send.rregSizePackets+=reply.getSize();
	     } 

  

	    NetMessage.Ip replyMsg = new NetMessage.Ip(reply, localAddr,
	      NetAddress.ANY, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
	      Constants.TTL_DEFAULT);
	    

		if(log.isInfoEnabled())
			log.info(JistAPI.getTime()+" Forward register "+ optRRep.getAddress(optRRep.getNumAddresses()-1)  +" to  ("+localAddr+")"); 
	    
	    JistAPI.sleep((long)(Math.random() * BROADCAST_JITTER));		
		
  	  for(int i=0;i<netEntitiesWired.length;i++)
  		  netEntitiesWired[i].send(replyMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
		
	    
	    //TODO : ajouter le ack à reply
		return  reply;
	}
	
	
	private void registreToBestAP(){
		
		
		
		//compute the best AP
	 	APTable.APEntry apCanditate=apTable.computeNearerAP();


	 	
	 	//not AP accesible
	 	if(apCanditate==null){
    		if(regReactif && !idle){
    			searchAP();
    		}
	 		return;
	 	}
	 	
	 	//apSelected is not in ap list=> not current ap
	 	if(apSelected!=null && !apTable.isInListAP(apSelected)){
	 		unregistreToAP(apSelected.addr);
	 		apSelected=null;
	 	}
	
	 	if(!idle){
			if(log.isInfoEnabled()){	
				log.info(apTable);
				log.info(JistAPI.getTime()+" AP candidate  ["+apCanditate+ "] Ap selected ["+ apSelected +"] ("+localAddr+")");
			}	 	
	 	}	 	
		//if the number of hop between the current and the candidate AP
		//is ( #TRAD# plus petit) 
		if(apSelected==null ||  (apSelected.hop-apCanditate.hop) >= 2 ){
			
			JistAPI.sleep((long)(Math.random() * REGISTER_JITTER));		
			
			//unregistre to the AP
			if(apSelected!=null && !apSelected.addr.equals(apCanditate.addr))
				unregistreToAP(apSelected.addr);			
			
			apSelected=apCanditate;

			if(log.isInfoEnabled())
				log.info((jist.runtime.JistAPI.getTime()+" Registre_to_the_AP "  + " " +apSelected+" ("+localAddr+")"));
			registreToAP(apSelected.addr,localAddr,apSelected.addrs,true);
		}
		
		
	}
	
	private void HandleAPAdvertissement(RouteDsrMsg msg,RouteDsrMsg.OptionRouteReply optRRep,RouteDsrMsg.OptionRouteRequest optRReq,RouteDsrMsg.OptionAccessPointAdvertissement optAPAdv,NetAddress src){

		if (log.isDebugEnabled())
 	        	log.debug("Receive a advertissement of  "+src+" ("+localAddr+")");

	 	if(!idle)
			if(log.isInfoEnabled())
				log.info(JistAPI.getTime()+" Receive a advertissement of  "+src+" ("+localAddr+")");

     	//compute the number of hop		
		int hop;

//		add the AP to the table
		if(optRReq!=null){
			hop=optRReq.getNumAddresses()+2;
			apTable.add(src, (byte)hop,optRReq);
		}
		
		if(optRRep!=null){
			hop=optRRep.getNumAddresses()+2;
			apTable.add(src, (byte)hop,optRRep);
		}
		if(!idle && !AP)
			registreToBestAP();
			
	}
	/**
		Same the super class, exepted not forward the RREQ
		called only if it's a AP
	*/
	  protected void HandleRequestAP(RouteDsrMsg msg, RouteDsrMsg.OptionRouteRequest opt,
              byte[] optBuf, NetAddress src, NetAddress dst,
              short protocol, byte priority, byte ttl,
              short id, short fragOffset)
	{

  	   if (log.isDebugEnabled())
 	        log.debug("AP receive a RREQ of "+src+"("+localAddr+")");

		// If this request came from this node, ignore it
		if (localAddr.equals(src)) return;
		
		// If we've seen this request lately, ignore it
		if (SeenRequestLately(src, opt.getId())) return;
		
		if (localAddr.equals(opt.getTargetAddress()))
		{
			// They're looking for this node, so send a reply
			SendRouteReply(opt, src);
		
		}else{
    	NetAddress[] cachedRoute = GetCachedRoute(opt.getTargetAddress());
    	
    	if(cachedRoute!=null){
            int numAdress = opt.getNumAddresses();
    	    NetAddress[] route = new NetAddress[numAdress + cachedRoute.length + 3];
    	    NetAddress[] routeFromHere = new NetAddress[numAdress];
    	    int i=0;
    	    route[i++] = src;
    	    for (int j=0; j < numAdress; j++)
    	    {
               routeFromHere[numAdress-j-1] = route[i++] = opt.getAddress(j);
    	    }    	   
    	    route[i++]=localAddr;
    	    
    	    for (int j=0; j < cachedRoute.length; j++)
    	    {
    	      route[i++] = cachedRoute[j];
    	    }
    	    
    	    route[i] = opt.getTargetAddress();
    	    
    	    SendSourceRouteReply(src,route,routeFromHere);    		
    	}else{
    		
//    	  Otherwise propagate the request in the wired interface
    	  for(int i=0;i<netEntitiesWired.length;i++)
	  	      ForwardRequest(netEntitiesWired[i],msg, opt, optBuf, src, dst, protocol,
	  	        priority, ttl, id, fragOffset);
    	}
    }

		
		
		
		// Make a note of this request in the route request table
		AddRequestId(src, opt.getId());
	}	
	  
	  
	//TODO a revoir : premier parse pour savoir les messages presents
	  protected void ProcessOptions(RouteDsrMsg msg, NetAddress src, NetAddress dst,
              short protocol, byte priority, byte ttl,
              short id, short fragOffset){
		  
		  
		  
		   Iterator iter = msg.getOptions().iterator();
		   RouteDsrMsg.OptionSourceRoute sourceRoute = null;
		   
		   /**True if can do it call ProcessOption of the super class*/
		   boolean callSuper=true;
		   
		   /**AP  options*/
		   
		    RouteDsrMsg.OptionAckRequest ackRequest = null;
	   
		   RouteDsrMsg.OptionAccessPointAdvertissement optApAdv=null;
		   RouteDsrMsg.OptionAccessPointSearch optApSearch=null;
		   RouteDsrMsg.OptionAccessPointRegistry optApReg=null;
		   //get the RouteRequest opiton for the AP advertissement option.
		   RouteDsrMsg.OptionRouteRequest optRReq=null;
		   
		   //get the RouteReply opiton for the AP advertissement option.
		   RouteDsrMsg.OptionRouteReply optRRep=null;	
		   
		   RouteDsrMsg.OptionRouteError optRerr=null;
		   
		   byte[] optBufRReq=null;
		   byte[] optBufSrcRoute=null;
		   
		    while (iter.hasNext())
		    {
		      byte[] optBuf = (byte[])iter.next();
		      RouteDsrMsg.Option opt = RouteDsrMsg.Option.create(optBuf, 0);

		      if (opt == null)
		      {
		        // This should never happen in the simulation
		        throw new RuntimeException("Unrecognized DSR Option");
		      }

		      switch (opt.getType())
		      {
		       case RouteDsrMsg.OPT_ROUTE_REQUEST:
		    	   optRReq=(RouteDsrMsg.OptionRouteRequest)opt;
		    	   optBufRReq=optBuf;

		  	   break;
		  	   
		       case RouteDsrMsg.OPT_ROUTE_REPLY:
		    	   optRRep=(RouteDsrMsg.OptionRouteReply)opt;
		    	   

		  	   break;		  	   
		  		  
		        case RouteDsrMsg.OPT_SOURCE_ROUTE:
		        
		        	sourceRoute = (RouteDsrMsg.OptionSourceRoute)opt;
		        	optBufSrcRoute = optBuf;

		            break;	
		            
		        case RouteDsrMsg.OPT_ROUTE_ERROR:
		        	optRerr=(RouteDsrMsg.OptionRouteError)opt;

		        	break;
		      
		        case RouteDsrMsg.OPT_AP_ADVERTISSEMENT:
 		        	if(AP)break;
		        	
	    			if(log.isDebugEnabled())
	    				log.debug(JistAPI.getTime()+"AP Advertissement from " + src + "("+ localAddr +")");
	
		            
		            optApAdv=(RouteDsrMsg.OptionAccessPointAdvertissement)opt;
		          break;
		          
		        case RouteDsrMsg.OPT_AP_SEARCH:
		        	
 		        	if(!AP)
 		        		break;
 		        	
		        	log.debug("AP Search from " + src + "("+ localAddr +")");
		        	

		            
		            optApSearch=(RouteDsrMsg.OptionAccessPointSearch)opt;
		          break;		        
		          
		        case RouteDsrMsg.OPT_AP_REGISTRY:
 		        	
		        	optApReg = (RouteDsrMsg.OptionAccessPointRegistry)opt;
		        	
		        	log.debug("AP Registry from " + src + "("+ localAddr +")");		        	
		        	

		            

		            
		          break;  
		          
		        case RouteDsrMsg.OPT_ACK_REQUEST:
		            ackRequest = (RouteDsrMsg.OptionAckRequest)opt;
		            
		            callSuper=false;

		            break;		          
		          
			    }
		      
		    }
		    

        	if(AP && sourceRoute!=null ){
	            boolean forward=true;
	            

		    //if i'am in path, forward
	            if (!localAddr.equals(NextRecipient(sourceRoute, dst)))
	            {
	              //find the best path shorting
	            	//TODO plutot obtenir the new route
	              int newSegLeft=PerformRouteShortening(sourceRoute, src, dst);
	              
	              /*if(newSegLeft>0){
	            	  
	        	      NetAddress[] route = new NetAddress[sourceRoute.getNumAddresses()];
	        	      for (int i = 0; i < route.length; i++)
	        	      {
	        	        route[i] = sourceRoute.getAddress(i);
	        	      }
	            	  sourceRoute = (RouteDsrMsg.OptionSourceRoute) RouteDsrMsg.Option.create(RouteDsrMsg.OptionSourceRoute.create(sourceRoute.isFirstHopExternal(),
	            			  sourceRoute.isLastHopExternal(), sourceRoute.getSalvageCount(), newSegLeft,
	            		        route), 0);
	   
	            	  
	              }else
	            	  forward=false;*/
	              
	              //TODO actualiser le segment left et forwarder
	              
	              
	            }	
	            
	            
	            if(forward)
	            {
	            	
	            	NetAddress nexthop= NextNextRecipient(sourceRoute,dst);
	            	NetAddress prevhop = Recipient(sourceRoute,src);
	            	

	            	
	            	//if the next node is mobile, forward the paquet in wireless interface
	            	if ((nexthop !=null) && !isNetworkAP(nexthop)){
	            		ForwardPacket(netEntityWireless,msg, sourceRoute, optBufSrcRoute, src, dst, protocol,
	            				priority, ttl, id, fragOffset);
	            	}
	            	

	            	//the previous node must receive the forward packet for passive ack
	            	if (!isNetworkAP(prevhop) ){
	            		ForwardPacketWithoutAck(netEntityWireless,msg, sourceRoute, optBufSrcRoute, src, dst, protocol,
	            				priority, ttl, id, fragOffset);
	            	}
	            	
  
            	
	            	
	            	//if the next node is a AP, forward without ack
	            	if ((nexthop !=null) &&  isNetworkAP(nexthop))
	            	{			            		
		            	NetInterface netEntityWired = getNetEntity(nexthop);
	            		
	            		if(netEntityWired!=null)
	            			ForwardPacketWithoutAck(netEntityWired,msg, sourceRoute, optBufSrcRoute, src, dst, protocol,
	            				priority, ttl, id, fragOffset);
	            	}
	            }

	            //if(optApReg==null)
	            	callSuper=false;
        	}

        	if(optRerr != null){
        	
        		HandleError(optRerr);
        		
	            if(stats!=null){
	            	  stats.recv.dsrPackets++;		            	
		          	  stats.recv.rerrPackets++;
		        }  		        	
	        	if(AP){
	        		HandleRouteErrorUnregistre(optRerr);
	        		//System.out.println(cacheToString());
	        	}else{
	        		checkAPEntryValid(optRerr);
	        	}
	        	
            	if(AP  && log.isDebugEnabled()){
            		
            		log.debug(this.localAddr + " Route error");
            		log.debug("Route Cache : "+cacheToString());
            		
            	} 	        	
	        	
	        	callSuper=false;
        	}

		    //Process the AP advertissement with the RREQ
		    if(optApAdv!=null && !AP){
		    
	        	if(optRReq==null && optRRep==null)
	        		throw new RuntimeException("AP advertissement without Route");

	        	if(optRRep==null || localAddr.equals(dst))
	        		HandleAPAdvertissement(msg,optRRep,optRReq,optApAdv,src);
	        	
	            if(stats!=null){
		          	  stats.recv.apAdvPackets++;
		        }  
		    }
		
		    
		    
		    if(optApSearch!=null && AP){
			    
	        	if(optRReq==null)
	        		throw new RuntimeException("Search requete AP without RREQ");
	        	
	        	HandleSearchAP(src,optRReq);
	            if(stats!=null){
	            	  stats.recv.dsrPackets++;
		          	  stats.recv.apSearchPackets++;
		        }  	
	        	callSuper=false;
		    }	
		    
		    if(optApSearch==null && AP && optRReq!=null){
				   //special process for a AP

	    		   HandleRequestAP(msg, optRReq, optBufRReq, src,dst, protocol, priority, ttl, id, fragOffset);
	                 
					if(stats!=null){
					  stats.recv.apSearchPackets++;
					  stats.recv.dsrPackets++; 	  
					}		
					//not forward the RREQ if AP
					callSuper=false;
		    	 	    	
		    }
		    
		    if(AP && optRRep!=null)
		    	callSuper=false;
		    


            if(optApReg!=null && AP && bbProactif && optRRep!=null){
            	
            	boolean forwardReg=true;
            	
	            if(stats!=null){            	
		          	  stats.recv.rregPackets++;
		        }  	      	

    			int nbr= optRRep.getNumAddresses();
    			
    			if(!optApReg.isForwardBB())
    				forwardReg=false;
    			else
	    			if(log.isInfoEnabled())
	    				log.info(JistAPI.getTime()+" Receive un APReg "+src+"("+ localAddr +")");
    			
    			//si on est dans le backbone et que la premiere adresse est la destination, ignore
    			if(localAddr.equals(optRRep.getAddress(0))&&optApReg.isInBackBone())
    				forwardReg=false;
    			
    			
    			if(forwardReg)
	    			for(int i=1;i<nbr;i++){
	    				
	    				//if the paquet is always passed here, ignore
	    				if(localAddr.equals(optRRep.getAddress(i))){
	    					forwardReg=false;
	    					
	    				}
	    				
	    				
	    			}
        			
    			if(forwardReg){
	    			log.debug("Forward AP Reg " + src + "("+ localAddr +")");
	
	    			msg=ForwardAPRegistry(msg,src,dst,optRRep,optApReg,sourceRoute);
	    			dst=localAddr; 
	    			
	    			//HandleReply(msg,optRRep);
	    			//callSuper=false;
	    			
	    		    NetAddress dest;
	    		    // Update the Route Request Table
	    		    dest = optRRep.getAddress(optRRep.getNumAddresses() - 1);

	    		    NetAddress[] route;
	    		    if(optRRep.getAddress(0).equals(localAddr)){
	    		        route = new NetAddress[optRRep.getNumAddresses() - 2];
	    		        for (int j = 0; j < route.length; j++)
	    		        {
	    		          route[j] = optRRep.getAddress(j+1);
	    		        }
	    		    }else{
	    		        route = new NetAddress[optRRep.getNumAddresses() - 1];
	    		        for (int j = 0; j < route.length; j++)
	    		        {
	    		          route[j] = optRRep.getAddress(j);
	    		        }	    		    	
	    		    }
    		        
    			  	if(log.isInfoEnabled()){
    					log.info(JistAPI.getTimeString()+" Insert_Route_AP "+ localAddr + " " + dest +" " +routeToString(route));
    				}        		        

    		        InsertRouteCache(dest, route);


	    			
    			}/*else
    			callSuper=false;
    				*/
            }	
            
            
            if(!AP && !callSuper && sourceRoute!=null){
                if (localAddr.equals(NextRecipient(sourceRoute, dst)))
                {
                  ForwardPacket(msg, sourceRoute, optBufSrcRoute, src, dst, protocol,
                    priority, ttl, id, fragOffset);
                }        	
            }
            
            if(/*!callSuper*/!AP && optRRep!=null){
            	

            	
                HandleReply(msg, optRRep);
                if(stats!=null){
              	  stats.recv.rrepPackets++;
              	  stats.recv.dsrPackets++; 	  
                }      
                
            	/*l'AP recoit un route registry*/
            	if(AP && optApReg!=null && log.isDebugEnabled()){
            		
            		log.debug(this.localAddr + " Registry");
            		log.debug("Route Cache : "+cacheToString());
            		
            	}                
                
            }
            
            //if the ackreq will not be process in the super class, process here
            if(!callSuper && ackRequest != null){
            	
	            if(stats!=null){
		          	  stats.recv.rackPackets++;
		          	  stats.recv.dsrPackets++; 	  
		            }               	
            	
	              // The DSR spec has some contradictory instructions regarding how to
	              // handle packets containing both acknowledgements and acknowledgement
	              // requests (in section 8.3.3).  It doesn't make any sense, so I'm
	              // ignoring those instructions.
	              HandleAckRequest(msg, ackRequest, src, dst, sourceRoute);
	           	             
            }
            
            if(!callSuper && msg.getContent()!=null){
                if(stats!=null){
              	  stats.recv.dataPackets++;
              	  stats.recv.dsrPackets++; 	  
                }   	
            	
            }
            
		  if(callSuper)
			  super.ProcessOptions(msg,src,dst,protocol,priority,ttl,id,fragOffset);
	  }
	
	
	protected void broadcastAdvertissement(){

  	   if (log.isDebugEnabled())
 	        log.debug("broadcast a AP advertissement "+localAddr+" at "+JistAPI.getTime());

  	   short reqestid=nextRequestId++;

  	 
  	   
		//Create the RREQ
	      RouteDsrMsg routeRequest = new RouteDsrMsg(null);
	      routeRequest.addOption(RouteDsrMsg.OptionRouteRequest.create(reqestid,
	    		  NetAddress.ANY, new NetAddress[0]));
	      
		//add the Access Point Option
	      routeRequest.addOption(RouteDsrMsg.OptionAccessPointAdvertissement.create());

	      if(stats!=null){	    	        		
	    	  stats.rreqOrig++;
	    	  /*stats.send.rreqPackets++;
	    	  stats.send.rreqSizePackets+=routeRequest.getSize();*/
	    	  stats.send.dsrPackets++;
	    	  stats.send.dsrSizePackets+=routeRequest.getSize();    
	    	  stats.send.apAdvPackets++;
	    	  stats.send.apAdvSizePackets+=routeRequest.getSize();	  
	      }

	      NetMessage.Ip routeRequestMsg = new NetMessage.Ip(routeRequest, localAddr,
	        NetAddress.ANY, Constants.NET_PROTOCOL_DSR, Constants.NET_PRIORITY_NORMAL,
	        diameter);
	      

	      //send the message
	      netEntityWireless.send(routeRequestMsg, Constants.NET_INTERFACE_DEFAULT, MacAddress.ANY);
		
		
		
	}
	
	
	  /**
	   * Determines the intended next next (next node) recipient of a packet with the given source
	   * route option and IP destination.
	   *
	   * @param sourceRoute the Source Route option (or <code>null</code> if none)
	   * @param dst         the destination IP address
	   * @return the address of the next recipient of the message.  If the Source
	   *   Route option is invalid in some way, <code>null</code> can be returned.
	   */
	  private NetAddress NextNextRecipient(RouteDsrMsg.OptionSourceRoute sourceRoute,
	                                   NetAddress dst)
	  {
	    if (sourceRoute == null)
	      return dst;

	    int curSegment = sourceRoute.getNumAddresses() - sourceRoute.getNumSegmentsLeft() +1;

	    if (curSegment < sourceRoute.getNumAddresses())
	    {
	      return sourceRoute.getAddress(curSegment);
	    }
	    else if (curSegment == sourceRoute.getNumAddresses())
	    {
	      return dst;
	    }
	    else
	    {
	      return null;
	    }
	  }	
	
	  
	  /**
	   * Determines the intended  recipient (previous node) of a packet with the given source
	   * route option and IP destination.
	   *
	   * @param sourceRoute the Source Route option (or <code>null</code> if none)
	   * @param src         the source IP address
	   * @return the address of the next recipient of the message.  If the Source
	   *   Route option is invalid in some way, <code>null</code> can be returned.
	   */
	  private NetAddress Recipient(RouteDsrMsg.OptionSourceRoute sourceRoute,
	                                   NetAddress src)
	  {
	    if (sourceRoute == null)
	      return src;

	    int prevSegment = sourceRoute.getNumAddresses() - sourceRoute.getNumSegmentsLeft() -1;

	    if (prevSegment >= 0)
	    {
	      return sourceRoute.getAddress(prevSegment);
	    }
	    else if (sourceRoute.getNumSegmentsLeft() == sourceRoute.getNumAddresses())
	    {
	      return src;
	    }
	    else
	    {
	      return null;
	    }
	  }	
	  
	  
	public boolean isNetworkAP(NetAddress addr){
    	//if addr had the same prefix that me, then addr is a AP
    	return (addr.getIP().getAddress()[0]==localAddr.getIP().getAddress()[0])		;
	}

    public void registerTimeOut(){

    	if(helloInter==0){
    		
    		JistAPI.sleep(Constants.SECOND);
        	self.registerTimeOut();
    	}
    	
		JistAPI.sleep((long)(Math.random() * helloInter));
		
		

			
			//send a register message hello
			if(apSelected!=null){
				registreToAP(apSelected.addr,localAddr,apSelected.addrs,false);
				if(!AP && !idle){
					if(log.isInfoEnabled())
						log.info(JistAPI.getTime()+" Register_Hello "+apSelected.addr+" "+localAddr+" "+routeToString(apSelected.addrs) );			
			}
			//registreToBestAP(false);
		}
    	
		JistAPI.sleep(helloInter);
    	self.registerTimeOut();
}	
	
	
    public void RequestPosTimeOut(){

    		//helloInter : arbitraire
    		JistAPI.sleep((long)(Math.random() * helloInter));
    			
    		if(regReactif && !AP  && !idle)
    			registreToBestAP();

    		
    		if(!regReactif && AP)
    			//broadcast the AP advsertissement at periodical period
    			//(mode proactif)
    			broadcastAdvertissement();
	    	
    		JistAPI.sleep(advertissementInter);
	    	self.RequestPosTimeOut();
    }
    
    /**
     * Sets the interface to the network layer.
     *
     * @param netEntityWired the interface to the network layer wired (AP network)
     * @param netEntityWireless the interface to the network layer wireless
     */
    public void addWirelessEntity(NetInterface netEntityWireless)
    {
    	
    	//this.netEntityWired=netEntityWired;
    	this.netEntityWireless=netEntityWireless;
    	
		netMI.addNetIp(netEntityWireless);    	
    	
    }
    
    /**
     * Add a wired interface to the AP
     * @param netEntity
     */
    public void addWiredEntity(NetInterface netEntity){
    	
    	NetInterface netEntityWired2[] = new NetInterface[netEntitiesWired.length+1];
    	
    	System.arraycopy(netEntitiesWired, 0, netEntityWired2, 0, netEntitiesWired.length);
    	netEntityWired2[netEntitiesWired.length]=netEntity;
    	
    	netEntitiesWired=netEntityWired2;
    	
    	netMI.addNetIp(netEntity);
    }
    
    /**
     * Add a routing table entry
     * @param addr
     * @param netEntity
     */
    public void addRoutingEntry(NetAddress addr,NetInterface netEntity){
    	
    	routingTable.put(addr, netEntity);
    	
    }  
    
    /**
     * 
     *  get the netEntity for addr
     * @param addr
     * @return the ip address
     */
    public NetInterface getNetEntity(NetAddress addr){
    	return (NetInterface)routingTable.get(addr);
    }

	public long getAdvertissementInter() {
		return advertissementInter;
	}

	 public void setStats(DsrStats stats)
	 {
	   super.setStats(stats);
	   this.stats = stats;
	 }  	
	
	public void setAdvertissementInter(long advertissementInter) {
		this.advertissementInter = advertissementInter;
		setRouteCacheEntryTimeOut((long)((double)advertissementInter*1.5));
	}

	public void setRegReactif() {
		this.regReactif = true;
	}

	public void setRegProactif() {
		this.regReactif = false;
	}

	public void setBbProactif() {
		this.bbProactif = true;
	}
	
	public void setBbReactif() {
		this.bbProactif = false;
	}

	public byte getDiameter() {
		return diameter;
	}

	public void setDiameter(byte diameter) {
		this.diameter = diameter;
	}

	public boolean isIdle() {
		return idle;
	}

	public void setIdle(boolean idle) {
		this.idle = idle;
	}

	public long getHelloInter() {
		return helloInter;
	}

	public void setHelloInter(long helloInter) {
		this.helloInter = helloInter;
	}	
	
}
