/**
 * 
 * Implement l'interface Placement, pour le placement des véhicule
 * initialement et pour l'insertion de nouveaux véhicules
 * 
 * On considère une loi de poisson pour le placement de véhicules
 * 
 * @author Florent Kaisser
 */

package jist.swans.runway;

import jist.swans.misc.Location;

public class Placement implements PlacementInterface {
	
	/**
	 * Placement pour une seule voie
	 */
	private class PlacementLan  {
		private RunWay rw;
		
		private float ecartVoiture; //ecart entre le dernier et l'avant dernier véhicule ajouté
		private float loc;//localisation du prochain véhicule à ajouter
		private int lane;
		
		
		public PlacementLan(RunWay rw,int lane) {
			this.rw=rw;
			this.lane=lane;
			
			nextEcart();
			
			reset();
		}
		
		//doit etre appelé avant chaque placement
		public void reset(){
			RunWayCar firstCar=rw.getFirstCar(lane);
			
			
			if(firstCar==null)
				this.loc=rw.getRwSize()-ecartVoiture;
			else
				this.loc=firstCar.loc-ecartVoiture;			
		
		}
		
		//calcule du nouvelle écart
		private void nextEcart(){
			ecartVoiture=(float)(1000.0*rw.rand.nextExponentiel((double)rw.getIntensity()));
			ecartVoiture+=(Constants.CAR_LENGTH+1);
		}

		public Location getNextLocation() {
			
			if(loc<0.0f){
				return null;
			}
			/*RunWayCar first=rw.getFirstCar(lane);
			if(first!=null)
				System.out.println(lane + " "+first.loc+" "+loc+ " " +ecartVoiture);*/
			Location l= new Location.Location2D(loc,(float)lane);
			
			nextEcart();
			loc-=ecartVoiture;
			
			
			return l;

		}		
	
	}
	
	
	private RunWay rw;
	private int currentLane;
	private PlacementLan[] plLane;

	/**
	 * placement pour toutes les voies
	 * 
	 * @param rw Autoroute su lequel on place les véhicules
	 */
	public Placement(RunWay rw) {
		this.rw=rw;
		
		plLane=new PlacementLan[rw.getNbrLan()];
		
		for (int i=0;i<rw.getNbrLan();i++){
			plLane[i]=new PlacementLan(rw,i);
		}		
		
		reset();

	}
	
	//doit etre appelé avant chaque placement
	public void reset(){
		currentLane=0;
		for (int i=0;i<rw.getNbrLan();i++){
			plLane[i].reset();
		}
	}

	public Location getNextLocation() {
		
		if(currentLane>=rw.getNbrLan())
			return null;
		
		Location l=plLane[currentLane].getNextLocation();
		
		if(l==null){
			currentLane++;
			return getNextLocation();
		}
		
		return l;
	}

}
