/**
 * Réprésente un véhicule
 * 
 * Gére ses propritétés (vitesse cible, longueur), 
 * et son déplacement (vitesse acceleration, changment de voie)
 * 
 * @author Florent Kaisser
 */

package jist.swans.runway;

import jist.swans.field.Mobility.MobilityInfo;
import jist.swans.misc.Location;
import jist.runtime.JistAPI;


public class RunWayCar implements MobilityInfo {

	public static class RunWayCarStats{
		public int nbrLaneChangement;//nombre de changement de voie
		private float somInterLaneChange;//somme du temps entre changement d'une voie
		private float somSpeed;//somme des vitesse 
		private float somAcc;//somme des acceleration 
		private float somGap;//somme des ecart avec la voiture de devant
		private float somDiffSpeed;//somme des difference de vitesse avec la voiture devant
		
		private long[] timeRegime;
		private long[] timeLane;
		
		public int nbStep;

		
		
		public float getMoyAcc() {
			return somAcc/nbStep;
		}

		public float getMoyDiffSpeed() {
			return somDiffSpeed/nbStep;
		}

		public float getMoyGap() {
			return somGap/nbStep;
		}

		public float getMoyInterLaneChange() {
			return somInterLaneChange/nbrLaneChangement;
		}

		public float getMoySpeed() {
			return somSpeed/nbStep;
		}

		public float getPartTimeLane(int lane) {
			return ((float)timeLane[lane])/nbStep;
		}

		public float getPartTimeRegime(byte regime) {
			return ((float)timeRegime[regime])/nbStep;
		}

		public void clear(RunWay rw){
		
			nbrLaneChangement=0;
			somInterLaneChange=0.0f;
			somSpeed=0.0f;
			somDiffSpeed=0.0f;
			somAcc=0.0f;
			somGap=0.0f;
			
			timeRegime=new long[Constants.NBR_REGIME];
			timeLane=new long[rw.getNbrLan()];
			
			
			for(int i=0;i<timeRegime.length;i++)
				timeRegime[i]=0;
			
			for(int i=0;i<timeLane.length;i++)
				timeLane[i]=0		;	
			
	
			
			nbStep=0;
	
		}
		
		public String toString(){
			String s="";
			
			s+="Nombre d'etape : " + nbStep+"\n";
			s+="Nombre de changment de voies "+nbrLaneChangement+"\n";
			s+="Temps moyen entre chaque changement de voie : "+getMoyInterLaneChange()+"\n";
			s+="Vitesse moyenne : "+getMoySpeed()+"\n";
			s+="Acceleration moyenne : "+getMoyAcc()+"\n";
			s+="Ecart moyen avec la voiture de devant : "+getMoyGap()+"\n";
			s+="Différence de vitesse moyenne avec la voiture de devant : "+getMoyDiffSpeed()+"\n";
			s+="Part de temps dans un regime : ";
			s+="Freeflow : "+getPartTimeRegime(Constants.REGIME_FREEFLOW)+ " ";
			s+="Car folowing : "+getPartTimeRegime(Constants.REGIME_CARFOLLOWING)+ " ";
			s+="Emergency :"+getPartTimeRegime(Constants.REGIME_EMERGENCY)+"\n";
			
			s+="Part de temps dans chaque voie : ";
			
			for (int i=0;i<timeLane.length;i++)
				s+="Voie "+i+" : "+getPartTimeLane(i)+ " ";
			s+="\n";
			
			
			return s;
		}
	}
	
	private RunWay rw;
	private long dateLastLaneChange=0;
	
	public Integer id;
	public float loc;//x location
	public int lane; //car's lan number
	//public boolean change; //true if the car has changed of lane during the slot
	//public boolean cluster; //true if the cluster had already been visited
	public float target; //target speed
	public float speed; //current speed
	public float acc; //current acceleration
	public byte regime; //current regime
	public float length;//véhicule's length

	RunWayCarStats stats;

	public RunWayCar prev,next;
	
	public RunWayCar(Integer id,RunWay rw,int lane,float loc){
		this.id=id;
		this.rw=rw;
		
		this.loc=loc;
		acc=0;
		//car.change=false;
		//car.cluster=false;
		this.lane=lane;
		regime=Constants.REGIME_FREEFLOW;
		
		length=Constants.CAR_LENGTH;
		
		//calcul la vitesse cible avec une loi normal centré sur la vitesse cible moyenne
		double mu=Constants.kmi[rw.getNbrLan()-1][lane]*rw.getAvgTargetSpeed();
		target=(float)rw.rand.nextNormal(mu, Constants.FACTEUR_DELTA_TARGET_SPEED*mu);
		speed=target;
		
		prev=next=null;
	}
	
	/**
	 * actualise l'acceleration courante de la voiture
	 * @return la nouvelle acceleration
	 */	
	public float updateAcceleration(){
		float h,diffV=0.0f,diffX=0.0f;
		
		if(next==null)
				h=7f; //car freefollow
				//si speed=0 ????
				//comment on inititalise diffX et diffV ?
		else{

				//calcul l'écart entre moi et la voiture que je suis
				diffX=next.loc-loc;
				//calcul l'ecrat de vitesse avec la voiture que je suis
				diffV=next.speed-speed;
				
				//si on est arreté
				if (speed<Constants.VITESSE_MIN)
					//on met la voiture en car following
					h=4.0f;
				else
					//calcul l'ecart de temps avec la voiture que je suis
					h=diffX/speed;
		}
		
		
		if (h<0.5){
			//Emergency regime
			//formule 3.10  MITSIM
			if(diffV<0.0f)
				acc=Math.min(Constants.decNorm(speed),next.acc - 0.5f*diffV*diffV/diffX);
			else
				acc=Math.min(Constants.decNorm(speed),next.acc + 0.25f*Constants.decNorm(speed));
			
			regime=Constants.REGIME_EMERGENCY;
		}else if(h<6.0f){
			//Car following regime
			if(diffV<0.0f){
				
				if((speed < target+Constants.DELTA_GOOD_TARGET_SPEED)&&(speed > target-Constants.DELTA_GOOD_TARGET_SPEED))
					acc=0.0f;
				else if(speed > target+Constants.DELTA_GOOD_TARGET_SPEED)
					acc=Constants.decNorm(speed);
				else 
					//formule 3.11 + Table C8 MITSIM
					acc=(float)(1.55*Math.pow(speed,1.08)*diffV/Math.pow(diffX,1.65));

			}else
				//formule 3.11 + Table C8 MITSIM
				acc = (float) (2.15*Math.pow(speed,-1.67)*diffV/Math.pow(diffX, -0.89));
					
			regime=Constants.REGIME_CARFOLLOWING;
		}else{
			//Freeflow regime
			
			
			if((speed < target+Constants.DELTA_GOOD_TARGET_SPEED)&&(speed > target-Constants.DELTA_GOOD_TARGET_SPEED))
				//la vitesse est proche de celle ciblée,on accelere pas
				acc=0.0f;
			else if(speed<target)
				acc=Constants.accMax(speed);
			else
				acc=Constants.decNorm(speed);
			
			regime=Constants.REGIME_FREEFLOW;
		}

		
		return acc;
	}
	
	/**
	 * retourne la valeur pour calculer les bonne conditions de changement de voie
	 * These Kazi formule 6.20
	 */
	private float valGoodCond(RunWayCar lead){
		return (float)(1.0/(1.0+Math.exp(2.08-0.0337*((double)(lead.speed-target))+0.152*(next.speed-speed)+0.0971*(lead.prev.speed-speed))));
	}	
	
	/**
	 * vérifie si c'est mieux de changer de voie
	 * 
	 * @return vrai si c'est mieux de changer de voie
	 */
	private boolean goodCond(RunWayCar lead){

		float u;
		
		if(next==null || lead==null || lead.prev==null)
			return false;

		u=rw.rand.nextFloat();

		return (u<valGoodCond(lead));		
		
		
		
	}
	
	/**
	 * retourne la voie où y a les meilleur condition
	 * 
	 * @return la voie gauche, droite ou aucune
	 */
	private int betterCondLan(RunWayCar leadL,RunWayCar leadR){

		float u;
		
		if(next==null || leadR==null || leadR.prev==null  || leadL==null || leadL.prev==null )
			return Constants.GOODCOND_NO;

		u=rw.rand.nextFloat();
		
		float gL=valGoodCond(leadL);
		float gR=valGoodCond(leadR);	
		
		if((u<gL)&&(gL>gR))
			return Constants.GOODCOND_LEFT;
		
		if((u<gR)&&(gR>gL))
			return Constants.GOODCOND_RIGHT;		
			
		return Constants.GOODCOND_NO;		
		
		
	}	
	
	/**
	 * retourne vrai si le véhicule à besoin de changer de voie
	 * 
	 * @return
	 */
	private boolean changeLan(){
		float gc,u;
		float delta=0.0f;
		
		//Si l'arriere du véhicule est à moins de 10m du véhicule qui me suit, delta tailgate est 1
		//These Kazi formule 6.17
		if( prev!=null && ((loc-prev.loc+length)<10.0f) ){
			delta=1.0f;
		}
			
		//These Kazi formule 6.19
		gc=(float)(1.0/(1.0+Math.exp(-0.225+0.0658*((double)(speed-target))-0.423*delta)));
		u=rw.rand.nextFloat();
			
		return (u<gc);
	}
	

	/**
	 * Vérifie si les conditions de sécurités sont bonne pour changer de voie, par rapport au véhicule qui suit le leader
	 * @param lead : leader
	 * @return vrai si les condition sont bonne
	 */
	private boolean goodLag(RunWayCar lead){
		float gc;
		float g;
		
		if(prev==null || lead==null || lead.prev==null)
			return false;	
		
		//différence de vitesse entre moi et celui qui suit le leader
		float diffV=lead.prev.speed-speed;
		
		/*	
		 * Calcul de écart critique entre moi et celui qui suit le leader
		 * These Kazi formule 6.22
		*/
		gc=(float)Math.exp(2.02+0.153* (double)Math.min(0.0f, diffV )+0.188*(double)Math.max(0.0f, diffV)+rw.rand.nextNormal(0.0, 0.276));
		
		//Calcul de écart entre moi et celui qui suit le leader
		g=loc-lead.prev.loc-length;
		
		return (g>gc);
		
	}

	/**
	 * Vérifie si les conditions de sécurités sont bonne pour changer de voie, par rapport au leader
	 * @param lead : leader
	 * @return vrai si les condition sont bonne
	 */	
	private boolean goodLead(RunWayCar lead){
		float gc;
		float g;
		
		if( lead==null)
			return false;
		
		/*
		 * Calcul de l'écart critique entre moi et  le leader
		 * These Kazi formule 6.21
		 */
		gc=(float)Math.exp(0.508-0.42*Math.min(0.0f,lead.speed-speed)+rw.rand.nextNormal(0.0, 0.238));
		
		//Calcul de l'écart entre moi et  le leader
		g=lead.loc-loc-length;
		
		return (g>gc);
	}	
	

	
	/**
	 * verifie si le véhicule doit changer de voie
	 * 
	 * @return le numéro de la nouvelle voie, -1 si y a pas eu de changement
	 */
	public int checkLan(){
		
		//si y a qu'une seul voie , on peut pas changer de voie
		if(rw.getNbrLan()<=1)
			return -1;
		
		//si on suit pas de voiture on change pas de voie (sauf si on est bouré)
		if(regime!=Constants.REGIME_CARFOLLOWING)
			return -1;
		
		//Si le véhicule n'a pas envis de changer de voie on fait rien
		if(!changeLan())
			return -1;
		
		
		if(lane==0){
			//voie de gauche
			RunWayCar carR=rw.getLeadR(lane, this);
			
			if(carR==null)
				return -1;
			
			//vérifie si on c'est mieux de changer de voie
			//et si les conditions de sécurité sont remplit
			if(goodCond(carR) && goodLag(carR) && goodLead(carR)){
				//on se met sur la voie à ma droite
				//lane=1;
				return 1;
			}
			
		}else if(lane==(rw.getNbrLan()-1)){
			//voie de droite
			RunWayCar carL=rw.getLeadL(lane, this);
			
			if(carL==null)
				return -1;
	
			//vérifie si on c'est mieux de changer de voie
			//et si les conditions de sécurité sont remplit			
			if(goodCond(carL) && goodLag(carL) && goodLead(carL)){
				//on se met sur la voie à ma gauche
				//lane=;
				return rw.getNbrLan()-2;
			}			
		}else{
			//voie centrale
			RunWayCar carL=rw.getLeadL(lane, this);
			RunWayCar carR=rw.getLeadR(lane, this);			
			
			if(carL==null || carR==null)
				return -1;
			
			//vérifie si on c'est mieux de changer de voie
			//si oui laquel ?
			switch (betterCondLan(carL, carR)) {
				case Constants.GOODCOND_LEFT:
					
					//vérifie si les conditions de sécurité sont remplit
					if (goodLag(carL) && goodLead(carL)){
						
						return lane-1;
						
					//peut être que à droite ca le fait mieux ?
					}else if(goodCond(carR) && goodLag(carR) && goodLead(carR)){
						
						return lane+1;
					}	
					
				case Constants.GOODCOND_RIGHT:
					//vérifie si les conditions de sécurité sont remplit
					if (goodLag(carR) && goodLead(carR)){
						
						return lane+1;
						
					//	peut être que à gauche ca le fait mieux ?
					}else if(goodCond(carL) && goodLag(carL) && goodLead(carL)){
						
						return lane-1;
					}		
			}
			
				
			
		}
		
		
		
		return -1;
	}
	
	/**
	 * Avance d'une étape
	 * actualise l'acceleration, sa voie, et fait avancer le véhicule
	 */
	public void nextStep(){
		float newSpeed,precision=rw.getPrecision();
			
		//si on a pas changé de voie recement, on vérifie si on doit changer de voie
		if((JistAPI.getTime()-dateLastLaneChange)>Constants.TEMPS_MIN_INTER_CHANGE_LAN){
			int newLane,oldLane;
			
			//On change de voie ?
			oldLane = lane;
			newLane = checkLan();
			
			if(newLane!=-1){
				
				//on change de voie
				lane=newLane;
				
				//on actualise les files
				rw.changeLane(this,oldLane,newLane);
				
				if(stats!=null){
					stats.nbrLaneChangement++;
					stats.somInterLaneChange+=(JistAPI.getTime()-dateLastLaneChange)/jist.swans.Constants.SECOND;
				}				
				
				//on enregitre le moment du dernier changement de voie
				dateLastLaneChange=JistAPI.getTime();
				

			}
		}
			
		
		//actualise l'accéleration
		updateAcceleration();
		
		//actualise la vitesse
		newSpeed=speed+acc*precision;			
		
		//vérifie si la vitesse n'est pas trop petite
		if(newSpeed>Constants.VITESSE_MIN){
			
			//actualise la vitesse
			speed=newSpeed;				
			
			//Calcul la nouvelle position
			loc+= precision *  ( 0.5*acc*precision + speed );
			
		
		}else{
			//Vitesse considéré comme nule, on bouge pas
			
			//on met la vitesse à la vitesse minimum considéré comme nulle
			speed=Constants.VITESSE_MIN;
		}
		
		//si on est trop près on se met à 1m et on adapte notre vitesse à la voiture qu'on suit
		if ((next!=null)&&((loc+length)>next.loc)){
			loc=next.loc-1.0f-length;
			speed=Math.min(speed,next.speed);
			//System.out.println(JistAPI.getTime() + " Freinage d'hyper urgence");
		}
		
		if(stats!=null){
			stats.nbStep++;
			stats.somAcc+=acc;
			if(next!=null){
				stats.somDiffSpeed+=(next.speed-speed);
				stats.somGap+=(next.loc-loc-length);
			}
			stats.somSpeed+=speed;
			stats.timeLane[lane]++;
			stats.timeRegime[regime]++;
		}
		
		
	}
	

	public String toString(){
		//String s= "" + ((int)loc);
		String s= "" + id + " [" + ((int)loc)+"]";		
		return s;
		
	}

	public void setStats(RunWayCarStats stats) {
		this.stats = stats;
	}
}
