package jist.swans.runway.graph;
import java.util.*;
import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.graph.filters.GeneralEdgeAcceptFilter;

public class Clusterise {
	protected Set<Cluster> clusters;
	protected Set<CarVertex> unvisited;	
	protected int sizeMax;
	protected GeneralEdgeAcceptFilter filter;


	public Clusterise(edu.uci.ics.jung.graph.Graph g){
		initialise(g);
		filter=null;
		sizeMax=Integer.MAX_VALUE;
	}
	
	
	protected void initialise(edu.uci.ics.jung.graph.Graph g){
		unvisited = new HashSet<CarVertex>(g.getVertices());
		clusters = new HashSet<Cluster>();
	}
	
	protected Cluster computeCluster(CarVertex v){
		ArrayList<CarVertex> currentList = new ArrayList<CarVertex>();
		Cluster cluster=new Cluster();
		
		//visit the first (the root) vertex;
		visitNewVertex(v,currentList,cluster);
		
	
		while(true){
			ArrayList<CarVertex> newList = new ArrayList<CarVertex>() ;
			
			for(Iterator<CarVertex> cIt=currentList.iterator();cIt.hasNext();){
				for (Iterator<Edge> eIt=cIt.next().getOutEdges().iterator();eIt.hasNext();){
					Edge e=eIt.next();
					
					if(filter==null || filter.acceptEdge(e)){
						visitNewVertex((CarVertex)e.getEndpoints().getSecond(),newList,cluster);
						if(cluster.size()>=sizeMax)return cluster;
					}
				}
			}
				
			if (newList.size() == 0) break;
			currentList=newList;
			
		}
		
		return cluster;
	}
	
	public void clusteriser(){
		
		while(unvisited.size()>0){
			//compute a cluster from the first vertex unvisited
			clusters.add(computeCluster(unvisited.iterator().next()));
		}

	}
	
	private void visitNewVertex(CarVertex v , List<CarVertex> newList, Cluster cluster){
		if(unvisited.contains(v)){
			cluster.add(v);
			newList.add(v);
			unvisited.remove(v);
		}
	}


	public void setFilter(GeneralEdgeAcceptFilter filter) {
		this.filter = filter;
	}


	public void setSizeMax(int sizeMax) {
		this.sizeMax = sizeMax;
	}


	public Set<Cluster> getClusters() {
		return clusters;
	}
	
}
