package jist.swans.runway.graph;

import java.util.Collection;
import java.util.Iterator;

import edu.uci.ics.jung.graph.Edge;

public class EdgeStat {
	
	public class Stats{
		double lifeTimeInBoundary;
		double lifeTimeMin;
		int nbEdge;
		
		public Stats(){
			nbEdge=0;
			lifeTimeInBoundary=0.0;
			lifeTimeMin=Double.MAX_VALUE;
		}
		
		public String toString(){
			return nbEdge+" "+lifeTimeMin +" " +lifeTimeInBoundary+ " ";
		}
	}

	private Collection<Edge> edges;
	private double ltBoundMin=0.0,ltBoundMax=0.0;
	private Stats s;
	
	static public float lifeTime(CarVertex  c1,CarVertex c2){
		float d = c2.getLoc()-c1.getLoc();
		
		if (d<0){
			CarVertex t=c1;
			c1=c2;
			c2=t;
			d = c2.getLoc()-c1.getLoc();
		}
		
		if (d<0)
			System.out.println("IMPOSSIBLE");
		
		float v = c2.getSpeed() - c1.getSpeed();
		
		
		if(v>0){
			return (Graph.DIST - d) / v;
		}else if (v<0){
			return (Graph.DIST + d) / -v;
		}
		
		return -1;
	}	
	
	static public float lifeTime(Edge e){
		return lifeTime((CarVertex)e.getEndpoints().getFirst(),(CarVertex)e.getEndpoints().getSecond());
	}
	
	public void setLifeTimeBoundary(double ltBoundMin,double ltBoundMax){
		this.ltBoundMin = ltBoundMin;
		this.ltBoundMax = ltBoundMax;
	}
	
	public EdgeStat(Collection<Edge> edges){
		this.edges=edges;
		this.s=new Stats();
	}
	
	public void computeStat(){

		int nbInBoundary=0;
		s.nbEdge=edges.size();
		for(Iterator<Edge>i=edges.iterator();i.hasNext();) {
			Edge e=i.next();
			double lt=(double)lifeTime(e);
			
			if(lt<s.lifeTimeMin)s.lifeTimeMin=lt;
			
			if(ltBoundMin  < lt &&  lt  < ltBoundMax){
				nbInBoundary++;
			}
		}
		
		s.lifeTimeInBoundary = (double)nbInBoundary/(double)s.nbEdge;
	}
	
	public Stats getStats(){
		return s;
	}
	
	
	
}
