package jist.swans.runway;

/**
 * 
 * Constantes de simulation 
 * 
 * @rem On considere que la classe de véhicule est la 1 : Voiture haute performence, avec passager (berline)
 * 
 * @author Florent Kaisser
 *
 */

public class Constants {
	
	
	public static final int NBR_REGIME				  = 3;
	public static final byte REGIME_FREEFLOW          = 0;
	public static final byte REGIME_CARFOLLOWING      = 1;
	public static final byte REGIME_EMERGENCY         = 2;
	
	public static final byte GOODCOND_LEFT			  = 1;
	public static final byte GOODCOND_RIGHT			  = 2;	
	public static final byte GOODCOND_NO			  = 0;		
	
	//temps minimum entre deux chamgement de voie (arbitraire)
	public static final long TEMPS_MIN_INTER_CHANGE_LAN = (long)(10*jist.swans.Constants.SECOND);
	
	//facteur pour générer aléatoirement une vitesse cible autour d'une viteese cible moyenne
	public static final float FACTEUR_DELTA_TARGET_SPEED = 0.05f;
	
	//marge d'erreur de la vitesse cible
	public static final float DELTA_GOOD_TARGET_SPEED = 0.83f;
	
	//longueur d'un véhicule
	public static final float CAR_LENGTH = 3.0f;
	
	
	//valeur de la vitesse minimum arbitraire
	public static final float VITESSE_MIN 		  = 0.001f;	
	
	
	/**
	 * différence de vitesse entre les voies
	 * Table C.6 MITSIM
	 */	
	public static final double[][] kmi={
			{1.0,0.0,0.0,0.0,0.0},
			{1.06,0.94,0.0,0.0,0.0},
			{1.06,1.01,0.93,0.0,0.0},
			{1.05,1.05,0.97,0.93,0.0},
			{1.04,1.06,1.01,0.95,0.94}
		};
	
	/**
	 * acceleration maximum (en m/s/s) en fonction de la vitesse
	 * Table C.2 MITSIM
	 */
	private static final float[] accMax={3.05f,2.41f,1.71f,1.22f};
	
	public static float accMax(float speed){
		int regime = (int)(speed/6.0f);
		
		if(regime<accMax.length)
			return accMax[regime];
		
		return accMax[accMax.length-1];
	}
	
	/**
	 * deceleration normal (en m/s/s) en fonction de la vitesse
	 * Table C.4 MITSIM
	 */
	private static final float[] decNorm={-2.38f,-2.04f,-1.46f};
	
	public static float decNorm(float speed){
		int regime = (int)(speed/6.0f);
		
		if(regime<decNorm.length)
			return decNorm[regime];
		
		return decNorm[decNorm.length-1];
	}	
}
